//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using Digitaldeposit.Ssbm.Common.Enums;

namespace Digitaldeposit.Ssbm.Common
{
	/// <remarks>
    ///  Used to check fo System objects like MessageTypes, Queue, Contracts, Logins etc.
    /// </remarks>
    static class SystemObjects
	{
		#region Fields
		static  string[] contracts = {
			"http://schemas.microsoft.com/SQL/Notifications/PostEventNotification",
			"http://schemas.microsoft.com/SQL/Notifications/PostQueryNotification",
			"http://schemas.microsoft.com/SQL/ServiceBroker/BrokerConfigurationNotice",
			"http://schemas.microsoft.com/SQL/ServiceBroker/ServiceDiagnostic",
			"http://schemas.microsoft.com/SQL/ServiceBroker/ServiceEcho",
			"DEFAULT"
		};
        
        
        static string[] MessageTypes ={
            "http://schemas.microsoft.com/SQL/ServiceBroker/Error",
			"http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog",
			"http://schemas.microsoft.com/SQL/Notifications/QueryNotification",
			"http://schemas.microsoft.com/SQL/Notifications/EventNotification",
			"http://schemas.microsoft.com/SQL/ServiceBroker/DialogTimer",
			"http://schemas.microsoft.com/SQL/ServiceBroker/BrokerConfigurationNotice/MissingRoute",
			"http://schemas.microsoft.com/SQL/ServiceBroker/BrokerConfigurationNotice/FailedRoute",
			"http://schemas.microsoft.com/SQL/ServiceBroker/BrokerConfigurationNotice/MissingRemoteServiceBinding",
			"http://schemas.microsoft.com/SQL/ServiceBroker/BrokerConfigurationNotice/FailedRemoteServiceBinding",
			"http://schemas.microsoft.com/SQL/ServiceBroker/ServiceEcho/Echo",
			"http://schemas.microsoft.com/SQL/ServiceBroker/ServiceDiagnostic/Query",
			"http://schemas.microsoft.com/SQL/ServiceBroker/ServiceDiagnostic/Status",
			"http://schemas.microsoft.com/SQL/ServiceBroker/ServiceDiagnostic/Description",
			"DEFAULT"
        };
        
        static string[] queues ={
            "QueryNotificationErrorsQueue",
			"EventNotificationErrorsQueue",
			"ServiceBrokerQueue"
        };

        static string[] services = {
			"http://schemas.microsoft.com/SQL/Notifications/EventNotificationService",
			"http://schemas.microsoft.com/SQL/Notifications/QueryNotificationService",
			"http://schemas.microsoft.com/SQL/ServiceBroker/ServiceBroker"
		};

		static string[] route = {
			"AutoCreatedLocal"
		};

        static string[] users = { 
			"dbo", 
			"guest", 
			"INFORMATION_SCHEMA", 
			"sys" 
		};

		static string[] logins = {
			"##MS_AgentSigningCertificate##",
			"NT AUTHORITY\\NETWORK SERVICE",
			"##MS_SQLAuthenticatorCertificate##" ,
			"##MS_SQLReplicationSigningCertificate##" , 
			"##MS_SQLResourceSigningCertificate##" ,
			"BUILTIN\\Administrators" , 
			"MS_AgentSigningCertificateLogin",
			"NT AUTHORITY\\SYSTEM" 
		};
		
		static string[] database = { 
			"master", 
			"model", 
			"msdb", 
			"tempdb" 
		};
		#endregion

		#region Public Static Methods
		/// <summary>
		/// Used to determine whether the corresponding object is a System defined object.
		/// </summary>
		/// <param name="ssbType">Type of object, example: Server, Database, Queue etc</param>
		/// <param name="name">Name of the object</param>
		/// <returns>Bool true or false</returns>
		internal static bool IsSystemSSBObject(SsbmObjectType ssbType, string name)
		{
			switch (ssbType)
			{
				case SsbmObjectType.MessageType:
					return ArrayContains(MessageTypes, name);
				case SsbmObjectType.Queue:
					return ArrayContains(queues, name);
				case SsbmObjectType.Contract:
					return ArrayContains(contracts, name);
				case SsbmObjectType.Service:
					return ArrayContains(services, name);
				case SsbmObjectType.Route:
					return ArrayContains(route, name);
				case SsbmObjectType.Login:
					return ArrayContains(logins, name);
				case SsbmObjectType.User:
					return ArrayContains(users, name);
				case SsbmObjectType.Database:
					return ArrayContains(database, name);
			}

			return false;
		}
     	static bool ArrayContains(string[] array, string value)
        {
            for (int i = 0; i < array.Length; i++)
            {
                if (array[i].CompareTo(value) == 0)
                {
                    return true;
                }
            }
            return false;
        }
        #endregion
    }
}
