//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.Specialized;
using System.Windows.Forms;
using System.Drawing;
using System.Xml;

using Digitaldeposit.Ssbm.Constants;

namespace Digitaldeposit.Ssbm.Common
{
	/// <summary>
	/// Utility Class to support this project
	/// </summary>
	static class Utility
	{
		#region Internal Static Functions
		/// <summary>
		/// Funtion to get all the SQL Keywords
		/// </summary>
		/// <returns>returns array of strings (SQL Keywords)</returns>
		internal static string[] GetSQLKeywords()
		{
			string[] keywords = {	
					"ADD",
					"EXCEPT",
					"PERCENT",
					"ALL",
					"EXEC",
					"PLAN",
					"ALTER",
					"EXECUTE",
					"PRECISION",
					"AND",
					"EXISTS",
					"PRIMARY",
					"ANY",
					"EXIT",
					"PRINT",
					"AS",
					"FETCH",
					"PROC",
					"ASC",
					"FILE",
					"PROCEDURE",
					"AUTHORIZATION",
					"FILLFACTOR",
					"PUBLIC",
					"BACKUP",
					"FOR",
					"RAISERROR",
					"BEGIN",
					"FOREIGN",
					"READ",
					"BETWEEN",
					"FREETEXT",
					"READTEXT",
					"BREAK",
					"FREETEXTTABLE",
					"RECONFIGURE",
					"BROWSE",
					"FROM",
					"REFERENCES",
					"BULK",
					"FULL",
					"REPLICATION",
					"BY",
					"FUNCTION",
					"RESTORE",
					"CASCADE",
					"GOTO",
					"RESTRICT",
					"CASE",
					"GRANT",
					"RETURN",
					"CHECK",
					"GROUP",
					"REVOKE",
					"CHECKPOINT",
					"HAVING",
					"RIGHT",
					"CLOSE",
					"HOLDLOCK",
					"ROLLBACK",
					"CLUSTERED",
					"IDENTITY",
					"ROWCOUNT",
					"COALESCE",
					"IDENTITY_INSERT",
					"ROWGUIDCOL",
					"COLLATE",
					"IDENTITYCOL",
					"RULE",
					"COLUMN",
					"IF",
					"SAVE",
					"COMMIT",
					"IN",
					"SCHEMA",
					"COMPUTE",
					"INDEX",
					"SELECT",
					"CONSTRAINT",
					"INNER",
					"SESSION_USER",
					"CONTAINS",
					"INSERT",
					"SET",
					"CONTAINSTABLE",
					"INTERSECT",
					"SETUSER",
					"CONTINUE",
					"INTO",
					"SHUTDOWN",
					"CONVERT",
					"IS",
					"SOME",
					"CREATE",
					"JOIN",
					"STATISTICS",
					"CROSS",
					"KEY",
					"SYSTEM_USER",
					"CURRENT",
					"KILL",
					"TABLE",
					"CURRENT_DATE",
					"LEFT",
					"TEXTSIZE",
					"CURRENT_TIME",
					"LIKE",
					"THEN",
					"CURRENT_TIMESTAMP",
					"LINENO",
					"TO",
					"CURRENT_USER",
					"LOAD",
					"TOP",
					"CURSOR",
					"NATIONAL",
					"TRAN",
					"DATABASE",
					"NOCHECK",
					"TRANSACTION",
					"DBCC",
					"NONCLUSTERED",
					"TRIGGER",
					"DEALLOCATE",
					"NOT",
					"TRUNCATE",
					"DECLARE",
					"NULL",
					"TSEQUAL",
					"DEFAULT",
					"NULLIF",
					"UNION",
					"DELETE",
					"OF",
					"UNIQUE",
					"DENY",
					"OFF",
					"UPDATE",
					"DESC",
					"OFFSETS",
					"UPDATETEXT",
					"DISK",
					"ON",
					"USE",
					"DISTINCT",
					"OPEN",
					"USER",
					"DISTRIBUTED",
					"OPENDATASOURCE",
					"VALUES",
					"DOUBLE",
					"OPENQUERY",
					"VARYING",
					"DROP",
					"OPENROWSET",
					"VIEW",
					"DUMMY",
					"OPENXML",
					"WAITFOR",
					"DUMP",
					"OPTION",
					"WHEN",
					"ELSE",
					"OR",
					"WHERE",
					"END",
					"ORDER",
					"WHILE",
					"ERRLVL",
					"OUTER",
					"WITH",
					"ESCAPE",
					"OVER",
					"WRITETEXT",
					"ABSOLUTE",
					"EXEC",
					"OVERLAPS",
					"ACTION",
					"EXECUTE",
					"PAD",
					"ADA",
					"EXISTS",
					"PARTIAL",
					"ADD",
					"EXTERNAL",
					"PASCAL",
					"ALL",
					"EXTRACT",
					"POSITION",
					"ALLOCATE",
					"FALSE",
					"PRECISION",
					"ALTER",
					"FETCH",
					"PREPARE",
					"AND",
					"FIRST",
					"PRESERVE",
					"ANY",
					"FLOAT",
					"PRIMARY",
					"ARE",
					"FOR",
					"PRIOR",
					"AS",
					"FOREIGN",
					"PRIVILEGES",
					"ASC",
					"FORTRAN",
					"PROCEDURE",
					"ASSERTION",
					"FOUND",
					"PUBLIC",
					"AT",
					"FROM",
					"READ",
					"AUTHORIZATION",
					"FULL",
					"REAL",
					"AVG",
					"GET",
					"REFERENCES",
					"BEGIN",
					"GLOBAL",
					"RELATIVE",
					"BETWEEN",
					"GO",
					"RESTRICT",
					"BIT",
					"GOTO",
					"REVOKE",
					"BIT_LENGTH",
					"GRANT",
					"RIGHT",
					"BOTH",
					"GROUP",
					"ROLLBACK",
					"BY",
					"HAVING",
					"ROWS",
					"CASCADE",
					"HOUR",
					"SCHEMA",
					"CASCADED",
					"IDENTITY",
					"SCROLL",
					"CASE",
					"IMMEDIATE",
					"SECOND",
					"CAST",
					"IN",
					"SECTION",
					"CATALOG",
					"INCLUDE",
					"SELECT",
					"CHAR",
					"INDEX",
					"SESSION",
					"CHAR_LENGTH",
					"INDICATOR",
					"SESSION_USER",
					"CHARACTER",
					"INITIALLY",
					"SET",
					"CHARACTER_LENGTH",
					"INNER",
					"SIZE",
					"CHECK",
					"INPUT",
					"SMALLINT",
					"CLOSE",
					"INSENSITIVE",
					"SOME",
					"COALESCE",
					"INSERT",
					"SPACE",
					"COLLATE",
					"INT",
					"SQL",
					"COLLATION",
					"INTEGER",
					"SQLCA",
					"COLUMN",
					"INTERSECT",
					"SQLCODE",
					"COMMIT",
					"INTERVAL",
					"SQLERROR",
					"CONNECT",
					"INTO",
					"SQLSTATE",
					"CONNECTION",
					"IS",
					"SQLWARNING",
					"CONSTRAINT",
					"ISOLATION",
					"SUBSTRING",
					"CONSTRAINTS",
					"JOIN",
					"SUM",
					"CONTINUE",
					"KEY",
					"SYSTEM_USER",
					"CONVERT",
					"LANGUAGE",
					"TABLE",
					"CORRESPONDING",
					"LAST",
					"TEMPORARY",
					"COUNT",
					"LEADING",
					"THEN",
					"CREATE",
					"LEFT",
					"TIME",
					"CROSS",
					"LEVEL",
					"TIMESTAMP",
					"CURRENT",
					"LIKE",
					"TIMEZONE_HOUR",
					"CURRENT_DATE",
					"LOCAL",
					"TIMEZONE_MINUTE",
					"CURRENT_TIME",
					"LOWER",
					"TO",
					"CURRENT_TIMESTAMP",
					"MATCH",
					"TRAILING",
					"CURRENT_USER",
					"MAX",
					"TRANSACTION",
					"CURSOR",
					"MIN",
					"TRANSLATE",
					"DATE",
					"MINUTE",
					"TRANSLATION",
					"DAY",
					"MODULE",
					"TRIM",
					"DEALLOCATE",
					"MONTH",
					"TRUE",
					"DEC",
					"NAMES",
					"UNION",
					"DECIMAL",
					"NATIONAL",
					"UNIQUE",
					"DECLARE",
					"NATURAL",
					"UNKNOWN",
					"DEFAULT",
					"NCHAR",
					"UPDATE",
					"DEFERRABLE",
					"NEXT",
					"UPPER",
					"DEFERRED",
					"NO",
					"USAGE",
					"DELETE",
					"NONE",
					"USER",
					"DESC",
					"NOT",
					"USING",
					"DESCRIBE",
					"NULL",
					"VALUE",
					"DESCRIPTOR",
					"NULLIF",
					"VALUES",
					"DIAGNOSTICS",
					"NUMERIC",
					"VARCHAR",
					"DISCONNECT",
					"OCTET_LENGTH",
					"VARYING",
					"DISTINCT",
					"OF",
					"VIEW",
					"DOMAIN",
					"ON",
					"WHEN",
					"DOUBLE",
					"ONLY",
					"WHENEVER",
					"DROP",
					"OPEN",
					"WHERE",
					"ELSE",
					"OPTION",
					"WITH",
					"END",
					"OR",
					"WORK",
					"END-EXEC",
					"ORDER",
					"WRITE",
					"ESCAPE",
					"OUTER",
					"YEAR",
					"EXCEPT",
					"OUTPUT",
					"ZONE",
					"EXCEPTION",
					"ABSOLUTE",
					"FOUND",
					"PREPARE",
					"ACTION",
					"FREE",
					"PRESERVE",
					"ADMIN",
					"FULLTEXTTABLE",
					"PRIOR",
					"AFTER",
					"GENERAL",
					"PRIVILEGES",
					"AGGREGATE",
					"GET",
					"READS",
					"ALIAS",
					"GLOBAL",
					"REAL",
					"ALLOCATE",
					"GO",
					"RECURSIVE",
					"ARE",
					"GROUPING",
					"REF",
					"ARRAY",
					"HOST",
					"REFERENCING",
					"ASSERTION",
					"HOUR",
					"RELATIVE",
					"AT",
					"IGNORE",
					"RESULT",
					"BEFORE",
					"IMMEDIATE",
					"RETURNS",
					"BINARY",
					"INDICATOR",
					"ROLE",
					"BIT",
					"INITIALIZE",
					"ROLLUP",
					"BLOB",
					"INITIALLY",
					"ROUTINE",
					"BOOLEAN",
					"INOUT",
					"ROW",
					"BOTH",
					"INPUT",
					"ROWS",
					"BREADTH",
					"INT",
					"SAVEPOINT",
					"CALL",
					"INTEGER",
					"SCROLL",
					"CASCADED",
					"INTERVAL",
					"SCOPE",
					"CAST",
					"ISOLATION",
					"SEARCH",
					"CATALOG",
					"ITERATE",
					"SECOND",
					"CHAR",
					"LANGUAGE",
					"SECTION",
					"CHARACTER",
					"LARGE",
					"SEQUENCE",
					"CLASS",
					"LAST",
					"SESSION",
					"CLOB",
					"LATERAL",
					"SETS",
					"COLLATION",
					"LEADING",
					"SIZE",
					"COMPLETION",
					"LESS",
					"SMALLINT",
					"CONNECT",
					"LEVEL",
					"SPACE",
					"CONNECTION",
					"LIMIT",
					"SPECIFIC",
					"CONSTRAINTS",
					"LOCAL",
					"SPECIFICTYPE",
					"CONSTRUCTOR",
					"LOCALTIME",
					"SQL",
					"CORRESPONDING",
					"LOCALTIMESTAMP",
					"SQLEXCEPTION",
					"CUBE",
					"LOCATOR",
					"SQLSTATE",
					"CURRENT_PATH",
					"MAP",
					"SQLWARNING",
					"CURRENT_ROLE",
					"MATCH",
					"START",
					"CYCLE",
					"MINUTE",
					"STATE",
					"DATA",
					"MODIFIES",
					"STATEMENT",
					"DATE",
					"MODIFY",
					"STATIC",
					"DAY",
					"MODULE",
					"STRUCTURE",
					"DEC",
					"MONTH",
					"TEMPORARY",
					"DECIMAL",
					"NAMES",
					"TERMINATE",
					"DEFERRABLE",
					"NATURAL",
					"THAN",
					"DEFERRED",
					"NCHAR",
					"TIME",
					"DEPTH",
					"NCLOB",
					"TIMESTAMP",
					"DEREF",
					"NEW",
					"TIMEZONE_HOUR",
					"DESCRIBE",
					"NEXT",
					"TIMEZONE_MINUTE",
					"DESCRIPTOR",
					"NO",
					"TRAILING",
					"DESTROY",
					"NONE",
					"TRANSLATION",
					"DESTRUCTOR",
					"NUMERIC",
					"TREAT",
					"DETERMINISTIC",
					"OBJECT",
					"TRUE",
					"DICTIONARY",
					"OLD",
					"UNDER",
					"DIAGNOSTICS",
					"ONLY",
					"UNKNOWN",
					"DISCONNECT",
					"OPERATION",
					"UNNEST",
					"DOMAIN",
					"ORDINALITY",
					"USAGE",
					"DYNAMIC",
					"OUT",
					"USING",
					"EACH",
					"OUTPUT",
					"VALUE",
					"END-EXEC",
					"PAD",
					"VARCHAR",
					"EQUALS",
					"PARAMETER",
					"VARIABLE",
					"EVERY",
					"PARAMETERS",
					"WHENEVER",
					"EXCEPTION",
					"PARTIAL",
					"WITHOUT",
					"EXTERNAL",
					"PATH",
					"WORK",
					"FALSE",
					"POSTFIX",
					"WRITE",
					"FIRST",
					"PREFIX",
					"YEAR",
					"FLOAT",
					"PREORDER",
					"ZONE",
					"COLLECTION",
					"PASSWORD", 
					"CHECK_EXPIRATION", 
					"CHECK_POLICY"
				};
			return keywords;
		
		}
		
		/// <summary>
		/// Function to get all SQL Service Broker Keywords
		/// </summary>
		/// <returns>string array of Keywords</returns>
		internal static string[] GetServiceBrokerKeywords()
		{
			string[] keywords = {	
					"MESSAGE",
					"TYPE",
					"VALIDATION",
					"VALID_XML",
					"CONTRACT",
					"SENT",
					"TARGET",
					"INITIATOR",
					"INITIATORTARGET",
					"QUEUE",
					"STATUS",
					"RETENTION",
					"ACTIVATION",
					"PROCEDURE_NAME",
					"MAX_QUEUE_READERS",
					"EXECUTE",
					"SERVICE",
					"ROUTE",
					"SERVICE_NAME",
					"BROKER_INSTANCE",
					"LIFETIME",
					"ADDRESS",
					"MIRROR_ADDRESS",
					"SCHEMA",
					"REMOTE",
					"BINDING",
					"BEGIN_DIALOG",
					"ACTIVE",
					"START_DATE",
					"EXPIRY_DATE", 
					"SUBJECT",
					"CERTIFICATE",
					"ENCRYPTION"
			};
			return keywords;
		}
		
		/// <summary>
		/// Function to format SQL Script in RichTextBox.
		/// </summary>
		/// <param name="line">SQL Script as string</param>
		/// <param name="rtb">Reference to RichTextControl</param>
		internal static void ParseLine(string line, ref RichTextBox rtb)
		{
			//Append spaces to line breaks and tab feeds, so that we can parse the token correctly
			line = line.Replace("\r", "\r ");
			line = line.Replace("\n", "\n ");
			
			char splitChar = ' ';
			String[] tokens = line.Split(splitChar);
			foreach (string token in tokens)
			{
				// Set the tokens default color and font.
				rtb.SelectionColor = Color.Black;
				rtb.SelectionFont = UIConstants.ViewScriptFont;

				// Set SQL Keywords to Blue. 
				String[] keywords = GetSQLKeywords();
				for (int i = 0; i < keywords.Length; i++)
				{
					if (keywords[i] == token.Trim())
					{
						// Apply alternative color and font to highlight keyword.
						rtb.SelectionColor = UIConstants.SQLKeywordsColor;
						rtb.SelectionFont = UIConstants.ViewScriptFont;
						break;
					}
				}

				//Set Service Broker Keywords to Pink
				keywords = GetServiceBrokerKeywords();
				for (int i = 0; i < keywords.Length; i++)
				{
					if (keywords[i] == token.Trim())
					{
						// Apply alternative color and font to highlight keyword.
						rtb.SelectionColor = UIConstants.ServiceBrokerKeyworksColor;
						rtb.SelectionFont = UIConstants.ViewScriptFont;
						break;
					}
				}
				rtb.SelectedText = token + splitChar.ToString();
			}
			rtb.SelectedText = "\n";
		}
		
		/// <summary>
		/// Function to get Script out of Script() method from SMO
		/// </summary>
		/// <param name="sc">Script() method return type StringCollection</param>
		/// <returns>SQL Script in the form of string</returns>
		internal static string GetScript(StringCollection sc)
		{
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < sc.Count; i++)
			{
				sb.AppendLine(sc[i]);
			}
			return sb.ToString();
		}
		#endregion
	}
}
