//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.Text;

namespace Digitaldeposit.Ssbm.Constants
{
	/// <summary>
	/// This SQL based UI application uses various SQL statements througout the project. 
	/// All the SQL Statements used are stored here as constant strings, sometime with parameter.
	/// </summary>
	static class SqlStatements
	{
		#region Internal Constants
		internal const string ENABLE_SERVICE_BROKER = "ALTER DATABASE {0} SET ENABLE_BROKER ";
		internal const string DISABLE_SERVICE_BROKER = "ALTER DATABASE {0} SET DISABLE_BROKER ";

		internal const string SERVICE_BROKER_STATUS_BY_DATABASE = "SELECT is_broker_enabled FROM SYS.DATABASES WHERE NAME='{0}'";
		internal const string DATABASE_PROPERTIES = @"SELECT 
															is_broker_enabled,
															is_master_key_encrypted_by_server 
														FROM 
															SYS.DATABASES WHERE NAME='{0}'";

		internal const string ALL_QUEUE_STATUS = @"SELECT 
													SQ.name as [QueueName],
													S.name as [ServiceName],
													SCH.name as [SchemaName], 
													CASE 
														WHEN DBQM.state is null THEN 'Not available' ELSE DBQM.state 
													END as [QueueState],  
													CASE 
														WHEN DBQM.tasks_waiting is null THEN '--' ELSE convert(varchar,DBQM.tasks_waiting) 
													END  as [TasksWaiting], 
													CASE
														WHEN DBQM.last_activated_time is null THEN '--' ELSE  convert(varchar,DBQM.last_activated_time) 
													END as [LastActivatedTime],  
													CASE
														when DBQM.last_empty_rowset_time is null THEN '--' ELSE convert(varchar,DBQM.last_empty_rowset_time) 
													END as [LastEmptyRowsetTime], 
													( SELECT count(*) FROM sys.transmission_queue TQ WHERE TQ.from_service_name = S.name) as [TranMessageCount] 
													FROM sys.services S 
													INNER JOIN sys.service_queues					SQ		on ( S.service_queue_id = SQ.object_id )   
													INNER JOIN sys.schemas							SCH		on ( SQ.schema_id = SCH.schema_id )  
													LEFT OUTER JOIN sys.dm_broker_queue_monitors	DBQM	on ( SQ.object_id = DBQM.queue_id  and DBQM.database_id = DB_ID() )  
													INNER JOIN sys.databases						DB		on DB.database_id = DB_ID()";

		internal const string QUEUE_STATUS_BY_QUEUENAME = ALL_QUEUE_STATUS + " and SQ.name = '{0}'";

		internal const string CONVERSATION_BY_ID = @"SELECT 
													CE.conversation_handle, 
													CE.conversation_group_id, 
													S.name	[ServiceName],  
													S.service_id, 
													SC.name [ContractName], 
													SQ.name [QueueName],
													CE.state_desc, 
													CE.far_service,
													CE.is_initiator,
													CE.lifetime,
													CE.far_broker_instance 
													FROM sys.conversation_endpoints	CE 
													JOIN sys.services				S	on CE.service_id = S.service_id 
													JOIN sys.service_contracts		SC	on CE.service_contract_id = SC.service_contract_id 
													JOIN sys.service_queues			SQ	on S.service_queue_id = SQ.object_id 
													WHERE CE.conversation_id = '{0}'";

		internal const string MESSAGES_IN_QUEUE = @"SELECT 
													message_type_name,
													message_sequence_number,
													queuing_order,
													conversation_handle,
													conversation_group_id,
													service_name,
													service_contract_name,
													CASE validation 
														WHEN 'E' THEN 'Empty'
														WHEN 'N' THEN 'None'
														WHEN 'X' THEN 'Xml'
														ELSE 'Unknown'
													END [validation],
													CASE status 
														WHEN 0 THEN 'Ready'
														WHEN 1 THEN 'Received message'
														WHEN 2 THEN 'Not yet complete'
														WHEN 3 THEN 'Retained sent message'
														ELSE 'Unknown'
													END [status],
													message_body 
													FROM {0} ";
		
		internal const string MESSAGES_IN_QUEUE_BY_CONVERSATIONHANDLE = MESSAGES_IN_QUEUE + @" WHERE conversation_handle = '{1}'";

		internal const string ALL_CONVERSATIONS_BY_ID = "SELECT DISTINCT conversation_id FROM SYS.CONVERSATION_ENDPOINTS";
		internal const string CONVERSATION_ID_BY_HANDLE = ALL_CONVERSATIONS_BY_ID + " WHERE conversation_handle = '{0}'";
		
		internal const string ALL_CONVERSATIONS_BY_HANDLE =  "SELECT DISTINCT conversation_handle FROM SYS.CONVERSATION_ENDPOINTS";

		
		internal const string END_CONVERSATION_BY_HANDLE = "END CONVERSATION '{0}' ";
		internal const string END_CONVERSATION_WITH_ERROR_BY_HANDLE = END_CONVERSATION_BY_HANDLE + " WITH ERROR = {1} DESCRIPTION = '{2}';";
		internal const string END_CONVERSATION_WITH_CLEANUP_BY_HANDLE = END_CONVERSATION_BY_HANDLE + " WITH CLEANUP;";

		internal const string MESSAGES_IN_TRANSMISSION_QUEUE = "SELECT * FROM SYS.transmission_queue";

		internal const string ACTIVATED_TASKS =		"SELECT * FROM SYS.dm_broker_activated_tasks";
		internal const string FORWARDED_MESSAGES =	"SELECT * FROM SYS.dm_broker_forwarded_messages";
		internal const string BROKER_CONNECTIONS =	"SELECT * FROM SYS.dm_broker_connections";
		internal const string QUEUE_MONITORS =		"SELECT * FROM SYS.dm_broker_queue_monitors";
		

		#endregion
	}
}
