//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//Comments: Bits and pieces of this file taken from public sources
//######################################################
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace Digitaldeposit.Ssbm.CustomControls
{
	/// <summary>
	/// There is no built in support for a ComboBox with images. 
	/// ComboBoxWithImages class that derives from ComboBox and allows 
	/// to set the ComboBoxWithImages.ImageList property and ComboBoxWithImagesItem.ImageIndex 
	/// property so that the ComboBox displays an image. 
	/// </summary>
	class ComboBoxWithImages : ComboBox
	{
		#region Fields
		private ImageList imageList;
		#endregion

		#region Properties
		public ImageList ImageList
		{
			get { return imageList; }
			set { imageList = value; }
		}
		#endregion

		#region Constructor
		public ComboBoxWithImages()
		{
			DrawMode = DrawMode.OwnerDrawFixed;
		}
		#endregion

		#region Events
		protected override void OnDrawItem(DrawItemEventArgs ea)
		{
			ea.DrawBackground();
			ea.DrawFocusRectangle();

			ComboBoxWithImagesItem item;
			Size imageSize;
			if(imageList != null)
				imageSize = imageList.ImageSize;
			else
				imageSize = new Size(16,16);
			
			Rectangle bounds = ea.Bounds;

			try
			{
				item = (ComboBoxWithImagesItem)Items[ea.Index];

				if (item.ImageIndex != -1)
				{
					imageList.Draw(ea.Graphics, bounds.Left, bounds.Top, item.ImageIndex);
					ea.Graphics.DrawString(item.Text, ea.Font, new SolidBrush(ea.ForeColor), bounds.Left + imageSize.Width, bounds.Top);
				}
				else
				{
					ea.Graphics.DrawString(item.Text, ea.Font, new SolidBrush(ea.ForeColor), bounds.Left, bounds.Top);
				}
			}
			catch
			{
				if (ea.Index != -1)
				{
					ea.Graphics.DrawString(Items[ea.Index].ToString(), ea.Font, new  SolidBrush(ea.ForeColor), bounds.Left, bounds.Top);
				}
				else
				{
					ea.Graphics.DrawString(Text, ea.Font, new SolidBrush(ea.ForeColor), bounds.Left, bounds.Top);
				}
			}
			
			base.OnDrawItem(ea);
		}
		#endregion
	}

}



