//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;

using Microsoft.SqlServer.Management.Smo.Broker;
using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.MessageBox;

using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.PropertyMaintenance;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;


namespace Digitaldeposit.Ssbm.DockableWindows
{
	/// <summary>
	/// Base window to display the properties of the selected object (queue, contract, service etc.)
	/// </summary>
	public partial class PropertiesWindow : UserControl
	{
		#region Constructor
		/// <summary>
		/// Constructor.
		/// </summary>
		public PropertiesWindow()
		{
			InitializeComponent();
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Called from Object Explorer to update the properties grid.
		/// </summary>
		/// <param name="server">SQL Server instance object</param>
		/// <param name="info">Info XmlNode which contains all the data related to the object.</param>
		/// <param name="db">Database or null if not known.</param>
		public void UpdatePropertiesGrid(Server server, XmlNode info, Database db)
		{

			//Background worker process accepts only one argument,
			//So, construct a array list with required arguments Info, Server and Databasee
			ArrayList bgwArgument = new ArrayList(3);
			bgwArgument.Add(info);
			bgwArgument.Add(server);
			bgwArgument.Add(db);
			
			//Kick off asynchronous call to populate properties.
			bgwPropSheetUpdater.RunWorkerAsync(bgwArgument);
		}
		#endregion

		#region Event Handlers
		private void bgwPropSheetUpdater_DoWork(object sender, DoWorkEventArgs e)
		{
			// This method will run on a thread other than the UI thread.
			// Be sure not to manipulate any Windows Forms controls created
			// on the UI thread from this method.
			if (bgwPropSheetUpdater.CancellationPending)
				e.Cancel = true;
			else
			{
				try
				{
					//Get the corresponding objects from the array list passed.
					ArrayList bgwArgument = (ArrayList)e.Argument;
					XmlNode info = (XmlNode)bgwArgument[0];
					Server server = (Server)bgwArgument[1];
					Database db = (Database)bgwArgument[2];

					//Set default Result as null
					e.Result = null;

					if (info != null)
					{
						NodeType nodeType = (NodeType)Enum.Parse(typeof(NodeType),info.Attributes.GetNamedItem(XmlNodeAttributeConstants.NODETYPE).InnerText);
						string ssbmObjectName = info.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText;
						if (nodeType == NodeType.Child)
						{
							SsbmObjectType ssbTypes = (SsbmObjectType)Enum.Parse(typeof(SsbmObjectType), info.Attributes.GetNamedItem(XmlNodeAttributeConstants.SSBTYPE).InnerText);

							switch (ssbTypes)
							{
								case SsbmObjectType.Server:
									e.Result = SsbmObjectTypeProperties.GetServerProperties(server);
									break;

								case SsbmObjectType.Database:
									e.Result = SsbmObjectTypeProperties.GetDatabaseProperties(db);
									break;

								case SsbmObjectType.EndPoint:
									Endpoint endPoint = null;
									lock(server.Endpoints.SyncRoot)
									{
										endPoint = server.Endpoints[ssbmObjectName];
									}

									if (endPoint != null)
									{
										e.Result = SsbmObjectTypeProperties.GetEndpointProperties(endPoint);
									}
									else
									{
										e.Result = null;
									}
									break;

								case SsbmObjectType.MessageType:
									MessageType mt = null;

									lock (db.ServiceBroker.MessageTypes.SyncRoot)
									{
										mt = db.ServiceBroker.MessageTypes[ssbmObjectName];
									}

									if (mt != null)
									{
										e.Result = e.Result = SsbmObjectTypeProperties.GetMessageTypeProperties(mt);
									}
									else
									{
										e.Result = null;
									}

									break;

								case SsbmObjectType.Contract:
									ServiceContract contract = null;
									
									lock(db.ServiceBroker.ServiceContracts.SyncRoot)
									{
										contract = db.ServiceBroker.ServiceContracts[ssbmObjectName];
									}

									if (contract != null)
									{
										e.Result = SsbmObjectTypeProperties.GetContractProperties(contract);
									}
									else
									{
										e.Result = null;
									}
									break;

								case SsbmObjectType.Queue:
									ServiceQueue queue = null;

									lock (db.ServiceBroker.Queues.SyncRoot)
									{
										queue = db.ServiceBroker.Queues[ssbmObjectName];
									}

									if (queue != null)
									{
										e.Result = SsbmObjectTypeProperties.GetQueueProperties(queue);
									}
									else
									{
										e.Result = null;
									}
									break;

								case SsbmObjectType.Service:
									BrokerService service = null;

									lock (db.ServiceBroker.Services.SyncRoot)
									{
										service = db.ServiceBroker.Services[ssbmObjectName];
									}

									if (service != null)
									{
										e.Result = SsbmObjectTypeProperties.GetServiceProperties(service);
									}
									else
									{
										e.Result = null;
									}
									break;

								case SsbmObjectType.Route:
									ServiceRoute route = null;

									lock (db.ServiceBroker.Routes.SyncRoot)
									{
										route = db.ServiceBroker.Routes[ssbmObjectName];
									}

									if (route != null)
									{
										e.Result = SsbmObjectTypeProperties.GetRouteProperties(route);
									}
									else
									{
										e.Result = null;
									}
									break;
								case SsbmObjectType.Certificate:
									Certificate cert = null;
									
									lock (db.Certificates.SyncRoot)
									{
										cert = db.Certificates[ssbmObjectName];
									}

									if (cert != null)
									{
										e.Result = SsbmObjectTypeProperties.GetCertificateProperties(cert);
									}
									else
									{
										e.Result = null;
									}
									break;
								case SsbmObjectType.RemoteBinding:
									RemoteServiceBinding rsb = null;

									lock (db.ServiceBroker.RemoteServiceBindings.SyncRoot)
									{
										rsb = db.ServiceBroker.RemoteServiceBindings[ssbmObjectName];
									}

									if (rsb != null)
									{
										e.Result = SsbmObjectTypeProperties.GetRemoteServiceBindingProperties(rsb);
									}
									else
									{
										e.Result = null;
									}
									break;
								case SsbmObjectType.User:
									User user = null;

									lock (db.Users.SyncRoot)
									{
										user = db.Users[ssbmObjectName];
									}

									if (user != null)
									{
										e.Result = SsbmObjectTypeProperties.GetDatabaseUserProperties(user);
									}
									else
									{
										e.Result = null;
									}
									break;

								case SsbmObjectType.SchemaCollection:
									XmlSchemaCollection schemaCol = null;

									lock (db.XmlSchemaCollections.SyncRoot)
									{
										schemaCol = db.XmlSchemaCollections[ssbmObjectName];
									}

									if (schemaCol != null)
									{
										e.Result = SsbmObjectTypeProperties.GetXmlSchemaCollectionProperties(schemaCol);
									}
									else
									{
										e.Result = null;
									}
									break;
								case SsbmObjectType.Login:
									Login login = null;
									
									lock(server.Logins.SyncRoot)
									{
										login = server.Logins[ssbmObjectName];
									}

									if (login != null)
									{
										e.Result = SsbmObjectTypeProperties.GetLoginProperties(login);
									}
									else
									{
										e.Result = null;
									}
									break;
								default:
									e.Result = null; break;
							}
						}
					}
				}
				catch (Exception ex)
				{
					e.Result = ex;
				}
			}
		}
		private void bgwPropSheetUpdater_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			if (!e.Cancelled)
			{
				if (e.Error == null) //This is not working properly, as documented in msdn.
				{
					object result = e.Result;

					Exception ex = result as Exception; //So, out exception handling mech.

					if (ex == null)
					{
						propertyGrid.SelectedObject = result;
					}
					else
					{
						propertyGrid.SelectedObject = null;
						
						ExceptionMessageBox emb = new ExceptionMessageBox(ex);
						emb.Show(this);
					}
				}
			}
		}
		#endregion
	}
}
