//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;

using DevComponents.DotNetBar;

using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.MessageBox;

using Digitaldeposit.Ssbm;
using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;
using Digitaldeposit.Ssbm.Forms;

namespace Digitaldeposit.Ssbm.DockableWindows
{
	/// <summary>
	/// User control to take care of Registered Servers window.
	/// </summary>
	public partial class RegisteredServersWindow : UserControl
	{
		#region Fields
		ObjectExplorerWindow g_ObjectExplorerWindow;
		ArrayList g_ObjectExplorerPopulatedServers;
		ArrayList g_ConnectedServers;
		#endregion

		#region Constructor
		/// <summary>
		/// Constructor
		/// </summary>
		public RegisteredServersWindow()
		{
			InitializeComponent();

			//Do basic setup
			Setup();
		}
		#endregion

		#region Private Methods
		private void Setup()
		{
			try
			{
				//Assign global ImageList to treeview
				tvRegisteredServers.ImageList = ImageListHelper.GetServiceBrokerImages();

				//Default root node
				TreeNode tnDatabaseEngine = new TreeNode(Digitaldeposit.Ssbm.Resources.ObjectExplorer.Treeview_DatabaseEngine);
				tnDatabaseEngine.ImageKey = ProjectImagesConstants.RegisteredServers;
				tnDatabaseEngine.SelectedImageKey = ProjectImagesConstants.RegisteredServers;
				tvRegisteredServers.ContextMenu = MenuHelper.ConstructRegServerContextMenu(tnDatabaseEngine, RegisteredServerTreeNode.DatabaseEngine, this);
				tvRegisteredServers.Nodes.Add(tnDatabaseEngine);

				//Load all the local instances into the treeview
				ArrayList localServers = Digitaldeposit.Ssbm.Manager.BrowseServers.GetServers(ServerLocation.Local);

				//Initialize g_ObjectExplorerPopulatedServers
				g_ObjectExplorerPopulatedServers = new ArrayList(localServers.Capacity);
				g_ConnectedServers = new ArrayList(localServers.Capacity);

				foreach (string s in localServers)
				{
					TreeNode tnServer = new TreeNode(s);
					tnServer.Tag = s;
					tnServer.Name = s;
					tnServer.ImageKey = ProjectImagesConstants.RegisteredServers;
					tnServer.SelectedImageKey = ProjectImagesConstants.RegisteredServers;
					tnServer.ContextMenu = MenuHelper.ConstructRegServerContextMenu(tnServer, RegisteredServerTreeNode.Server, this);
					tvRegisteredServers.Nodes[0].Nodes.Add(tnServer);
				}

				//Show everything.
				tvRegisteredServers.Nodes[0].ExpandAll();
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		private bool Connect(TreeNode tn)
		{
			try
			{
				TreeNode selectedNode = tn;
				tvRegisteredServers.SelectedNode = tn;

				if (g_ConnectedServers.Contains(selectedNode.Name) == false) //Show Login screen only if connection is not established.
				{
					//Change selected node text to inform connection on progress
					string originalText = tn.Text;
					tn.Text = tn.Text + Resources.RegisteredServers.Treeview_ConnectingStatus;

					//Make a connection to the selected Server
					Digitaldeposit.Ssbm.Screens.LogonForm formLogin = new Digitaldeposit.Ssbm.Screens.LogonForm(selectedNode.Name);
					if (formLogin.ShowDialog() == DialogResult.OK)
					{
						Server server = formLogin.ConnectedServer;
						if (server != null)
						{
							tn.Tag = server;
							
							g_ConnectedServers.Add(server.Name);
							
							//Add connected server to Object Explorer.
							PopulateObjectBrowser(server, true);

							//Make UI Menu changes
							SetMenuAccessibility(tn, true);
						}
						else
						{
							//Connection failure
							tn.Text = originalText;
							return false;
						}
					}
					else 
					{
						tn.Text = originalText;
						return false;
					}
					//Set the text back to original text
					tn.Text = originalText;
				}
			
				return true;
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
				return false;
			}
		}
		private bool Disconnect(TreeNode tn)
		{
			try
			{
				TreeNode selectedNode = tn;
				tvRegisteredServers.SelectedNode = tn;

				if (g_ConnectedServers.Contains(selectedNode.Name)) //Disconnect only if its already connected
				{
					Server server = this.GetServerFromRegisteredServers(selectedNode.Name);
					server.ConnectionContext.Disconnect();

					//Maintain Connected servers array list
					g_ConnectedServers.Remove(selectedNode.Name);

					//Clear the Treenode tag, or else server details will be cached 
					tn.Tag = null;

					//Clear Disconnected server from object explorer.
					PopulateObjectBrowser(server, false);

					//Set up UI Menu items accessibility
					SetMenuAccessibility(tn, false);
				

					return true;
				}

				return false;
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
				return false;
			}
		}
		private void PopulateObjectBrowser(Server server, bool addServer)
		{
			try
			{
				//Find the object explorer window container from the Main form and call UpdateTreeView method
				Mainform mainForm = this.ParentForm as Mainform;
				DevComponents.DotNetBar.DotNetBarManager dnm = mainForm.DotNetBarManager;
				DockContainerItem dockItem = dnm.GetItem("dockObjectExplorer") as DockContainerItem;
				g_ObjectExplorerWindow = dockItem.Control as ObjectExplorerWindow;

				//Asynchronous call to update object explorer
				if (g_ObjectExplorerPopulatedServers.Contains(server.Name) == false && addServer == true)
				{
					g_ObjectExplorerWindow.UpdateTreeView(server, true);
					g_ObjectExplorerPopulatedServers.Add(server.Name);
				}
				else if (g_ObjectExplorerPopulatedServers.Contains(server.Name) == true && addServer == false)
				{
					g_ObjectExplorerWindow.UpdateTreeView(server, false);
					g_ObjectExplorerPopulatedServers.Remove(server.Name);
				}
				dockItem.Refresh();
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		#endregion

		#region Private Static Methods
		private static void SetMenuAccessibility(TreeNode tn, bool isConnected)
		{
			if (tn != null)
			{
				MenuItem[] miCtxConnect = tn.ContextMenu.MenuItems.Find(MenuNameConstants.MenuRegisteredServersConnect, true);
				MenuItem[] miCtxDisconnect = tn.ContextMenu.MenuItems.Find(MenuNameConstants.MenuRegisteredServersDisconnect, true);


				if (miCtxConnect.Length > 0 && miCtxDisconnect.Length > 0)
				{
					if (isConnected)
					{
						miCtxConnect[0].Enabled = false;
						miCtxDisconnect[0].Enabled = true;
					}
					else
					{
						miCtxConnect[0].Enabled = true;
						miCtxDisconnect[0].Enabled = false;
					}
				}
			}
		}
		#endregion

		#region Internal / Public Methods
		/// <summary>
		/// Used to get the corresponding SQL Server instance object from server name.
		/// </summary>
		/// <param name="serverName">name of the server</param>
		/// <returns>Corrensponding Server object.</returns>
		internal Server GetServerFromRegisteredServers(string serverName)
		{
			try
			{
				TreeNode[] tnc = tvRegisteredServers.Nodes.Find(serverName, true);

				if (tnc.Length > 0)
					return (Server)tnc[0].Tag;
				else
					return null;
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
				return null;
			}
		}
		#endregion

		#region Event Handlers
		internal void ContextMenuHandler(object sender, EventArgs e)
		{
			try
			{
				MenuItem mnu = (MenuItem)sender;
				TreeNode tn = null;
				switch (mnu.Name)
				{
					case MenuNameConstants.MenuRegisteredServersConnect:
						tn = (TreeNode)mnu.Tag;
						Connect(tn);
						break;
					case MenuNameConstants.MenuRegisteredServersDisconnect:
						tn = (TreeNode)mnu.Tag;
						Disconnect(tn);
						break;
					case MenuNameConstants.MenuRegisteredServersBrowseServers:
						break;
				}
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		#endregion
	}
}
