﻿//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//Comments: This file is taken from Microsoft SQL 2005 Samples and modified
//######################################################
#region Using directives

using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Transactions;

using Microsoft.SqlServer.Management.Smo;

using Digitaldeposit.Ssbm.Constants;

#endregion

namespace Digitaldeposit.Ssbm.Manager
{
	/// <remarks>
	/// The <c>Conversation</c> class represents a service broker conversation between
	/// services. The convesation object may be obtained by invoking <c>Service.GetConversation</c>
	/// or <c>Service.BeginDialog</c> or by invoking the constructor and attaching it to 
	/// a <c>Service</c> object. Messages may be sent or received by calling the <c>Send</c> and
	/// <c>Receive</c> methods.
	/// </remarks>
	public class Conversation
	{
		#region Fields
		private Guid _handle;
		private Guid _groupHandle;
		private DateTime _lifeTime;
		private string _queueName;
		private string _contractName;
		private string _serviceName;
		private string _stateDescription;
		private MessageCollection _messages;
		private bool _isInitiator;
		private Database _database;
		#endregion

		#region Properties

		/// <summary>
		/// Holds the collection of Messages in the Conversation.
		/// </summary>
		public MessageCollection Messages
		{
			set { _messages = value; }
			get { return _messages; }
		}

		/// <summary>
		/// Guid representing Conversation Handle
		/// </summary>
		public Guid ConversationHandle
		{
			set { _handle = value; }
			get { return _handle; }
		}

		/// <summary>
		/// Guid representing Conversation Group Handle
		/// </summary>
		public Guid ConversationGroupHandle
		{
			set { _groupHandle = value; }
			get { return _groupHandle; }
		}

		/// <summary>
		/// Life time of the Conversation.
		/// </summary>
		public DateTime LifeTime
		{
			set { _lifeTime = value; }
			get { return _lifeTime; }
		}

		/// <summary>
		/// Service Name
		/// </summary>
		public string ServiceName
		{
			set { _serviceName = value; }
			get { return _serviceName; }
		}

		/// <summary>
		/// Contract used in the Conversation.
		/// </summary>
		public string ContractName
		{
			set { _contractName = value; }
			get { return _contractName; }
		}

		/// <summary>
		/// Queue used by the Service in the Conversation.
		/// </summary>
		public string QueueName
		{
			set { _queueName = value; }
			get { return _queueName; }
		}

		/// <summary>
		/// Explains the state of the Conversation.
		/// </summary>
		public string StateDescription
		{
			set { _stateDescription = value; }
			get { return _stateDescription; }
		}

		/// <summary>
		/// Indicates whether this Conversation is on the Initiator side or Target side.
		/// </summary>
		public bool IsInitiator
		{
			set { _isInitiator = value; }
			get { return _isInitiator; }
 
		}

		/// <summary>
		/// Database where the Conversation is taking place.
		/// </summary>
		public Database Database
		{
			set { _database = value; }
			get { return _database; }

		}

		#endregion

		#region Constructors
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="database">Database where the Conversation is taking place.</param>
		/// <param name="conversationHandle">Guid representing Conversation Handle</param>
		/// <param name="conversationGroupHandle">Guid representing Conversation Group Handle</param>
		/// <param name="lifeTime">Life time of the Conversation.</param>
		/// <param name="queueName"> Queue used by the Service in the Conversation.</param>
		/// <param name="contractName">Contract used in the Conversation.</param>
		/// <param name="serviceName">Service Name</param>
		/// <param name="stateDescription">Explains the state of the Conversation.</param>
		public Conversation(Database database, Guid conversationHandle, Guid conversationGroupHandle, DateTime lifeTime, string queueName, string contractName, string serviceName, string stateDescription)
		{
			_database = database;
			_handle = conversationHandle;
			_groupHandle = conversationGroupHandle;
			_lifeTime = lifeTime;
			_queueName = queueName;
			_contractName = contractName;
			_serviceName = serviceName;
			_stateDescription = stateDescription;
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="conversationHandle">Guid representing Conversation Handle</param>
		public Conversation(Guid conversationHandle)
		{
			_handle = conversationHandle;
		}
		/// <summary>
		/// Constructor.
		/// </summary>
		public Conversation()
		{
			
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Send the <paramref>message</paramref> on this conversation to
		/// remote service(s). The message is not actually sent until the transaction is committed
		/// </summary>
		/// <param name="message">The message to be sent. It should have a message type. Body is optional.</param>
		public void Send(Message message)
		{
			using(TransactionScope tranScope = new TransactionScope())
			{
				SqlParameter param;
				SqlConnection conn = ServiceBrokerManager.CreateDatabaseConnection(_database);
				SqlCommand cmd = conn.CreateCommand();

				string query = "SEND ON CONVERSATION @ch MESSAGE TYPE @mt ";
				param = cmd.Parameters.Add("@ch", SqlDbType.UniqueIdentifier);
				param.Value = _handle;
				param = cmd.Parameters.Add("@mt", SqlDbType.NVarChar, 255);
				param.Value = message.Type;

				if (message.Body != null)
				{
					query += " (@msg)";
					param = cmd.Parameters.Add("@msg", SqlDbType.VarBinary, -1);
					param.Value = new SqlBytes(message.Body);
				}
				cmd.CommandText = query;
				cmd.ExecuteNonQuery();

				//Complete Transaction
				tranScope.Complete();
			}
		}

		/// <summary>
		/// Receive a message from the fetched batch of messages from the queue.
		/// </summary>
		/// <returns>The <c>Message</c> object with all properties set.</returns>
		public Message Receive()
		{
			return null;// m_svc.Reader.Read(this);
		}

        /// <summary>
        /// Removes all the remaining messages associated with this conversation 
        /// from the underlying MessageReader.
        /// </summary>
        public void DrainReader()
        {
            try
            {
                Message msg = null;
                do
                {
					//TODO:
                    //msg = m_svc.Reader.Read(this);
                } while (msg != null);
            }
            catch
            {
                // this is a temporary hack to deal with a bug in the client stack
                // where it throws an exception upon close of a datareader following
                // a rollback.  This appears to happen only if the rollback occurs prior
                // to processing all of the existing messages in the datareader.
                ;
            }
        }

        /// <summary>
		/// Move this conversation to a new group by invoking the 'MOVE CONVERSATION' T-SQL
		/// command. Changes are not reflected until the transaction commits
		/// </summary>
		/// <param name="newGroupId">The new conversation group ID</param>
		public void MoveToGroup(Guid newGroupId)
		{
			using(TransactionScope tranScope = new TransactionScope())
			{
				SqlParameter param;
				SqlConnection conn = ServiceBrokerManager.CreateDatabaseConnection(_database);
				SqlCommand cmd = conn.CreateCommand();

				cmd.CommandText = "MOVE CONVERSATION @ch TO @cgid";
				param = cmd.Parameters.Add("@ch", SqlDbType.UniqueIdentifier);
				param.Value = _handle;
				param = cmd.Parameters.Add("@Cgid", SqlDbType.UniqueIdentifier);
				param.Value = newGroupId;
				cmd.ExecuteNonQuery();

				//Complete Transaction
				tranScope.Complete();
			}
		}

		/// <summary>
		/// End the conversation by invoking the 'END' T-SQL command.
		/// Changes are not reflected until the transaction commits
		/// </summary>
		public virtual void End()
		{
			using(TransactionScope tranScope = new TransactionScope())
			{
				SqlParameter param;
				SqlConnection conn = ServiceBrokerManager.CreateDatabaseConnection(_database);
				SqlCommand cmd = conn.CreateCommand();
				
				cmd.CommandText = "END CONVERSATION @ch";
				param = cmd.Parameters.Add("@ch", SqlDbType.UniqueIdentifier);
				param.Value = _handle;
				cmd.ExecuteNonQuery();

				DrainReader();

				//Complete Transaction
				tranScope.Complete();
			}
        }

		/// <summary>
		/// End the conversation with clean-up (Hard kill). This should not be used in
		/// typical applications.
		/// Changes are not reflected until the transaction commits
		/// </summary>
		public virtual void CleanUp()
		{
			using(TransactionScope tranScope = new TransactionScope())
			{
				SqlParameter param;
				SqlConnection conn = ServiceBrokerManager.CreateDatabaseConnection(_database);
				SqlCommand cmd = conn.CreateCommand();
				
				cmd.CommandText = "END CONVERSATION @ch WITH CLEANUP";
				param = cmd.Parameters.Add("@ch", SqlDbType.UniqueIdentifier);
				param.Value = _handle;
				cmd.ExecuteNonQuery();

				DrainReader();

				//Complete Transaction
				tranScope.Complete();
			}
        }

		/// <summary>
		/// End the conversation with an error. This should not be used in
		/// typical applications.
		/// Changes are not reflected until the transaction commits
		/// </summary>
		/// <param name="errorCode">An integer representing the error code.</param>
		/// <param name="errorDescription">A text description of the error.</param>
		public virtual void EndWithError(
			int errorCode,
			string errorDescription)
		{
			SqlParameter param;
			
			using(TransactionScope tranScope = new TransactionScope())
			{
				SqlConnection conn = ServiceBrokerManager.CreateDatabaseConnection(_database);
				SqlCommand cmd = conn.CreateCommand();
				
				cmd.CommandText = "END CONVERSATION @ch WITH ERROR = @ec DESCRIPTION = @desc";
				param = cmd.Parameters.Add("@ch", SqlDbType.UniqueIdentifier);
				param.Value = _handle;
				param = cmd.Parameters.Add("@ec", SqlDbType.Int);
				param.Value = errorCode;
				param = cmd.Parameters.Add("@desc", SqlDbType.NVarChar, 255);
				param.Value = errorDescription;
				cmd.ExecuteNonQuery();

				DrainReader();

				//Complete Transaction
				tranScope.Complete();
			}

        }
		#endregion

		#region Static Methods
		/// <summary>
		/// It invokes the BEGIN DIALOG T-SQL command and creates a new
		/// Conversation object wrapping the returned conversation handle./// </summary>
		/// <param name="database">Name of the Database</param>
		/// <param name="fromServiceName">From Service Name.</param>
		/// <param name="toServiceName">To Service Name</param>
		/// <param name="contractName">Contract Name</param>
		/// <param name="lifetime">Life time of the Conversation.</param>
		/// <param name="encryption">Whether to use Encryption.</param>
		/// <param name="relConversationHandle">If the conversation is going to join an existing one with same Conversation handle.</param>
		/// <param name="relConversationGroupHandle">If the conversation is going to join an existing one with same Conversation Group handle.</param>
		/// <returns></returns>
		internal static Guid BeginDialog(
			Database database,
			string fromServiceName,
			string toServiceName,
			string contractName,
			TimeSpan lifetime,
			bool encryption,
			Guid relConversationHandle,
			Guid relConversationGroupHandle)
		{
			Guid conversationHandle = Guid.Empty;
			
			try
			{
				using (System.Transactions.TransactionScope tranScope = new System.Transactions.TransactionScope())
				{

					SqlConnection conn = ServiceBrokerManager.CreateDatabaseConnection(database);
					SqlCommand cmd = conn.CreateCommand();
					SqlParameter param;
			
					StringBuilder query = new StringBuilder("BEGIN DIALOG @ch FROM SERVICE @fs TO SERVICE @ts ON CONTRACT @cn WITH ENCRYPTION = ");
					if (encryption)
						query.Append("ON ");
					else
						query.Append("OFF ");
					if (relConversationHandle != Guid.Empty)
					{
						query.Append(", RELATED_CONVERSATION = @rch ");
						param = cmd.Parameters.Add("@rch", SqlDbType.UniqueIdentifier);
						param.Value = relConversationHandle;
					}
					else if (relConversationGroupHandle != Guid.Empty)
					{
						query.Append(", RELATED_CONVERSATION_GROUP = @rcg ");
						param = cmd.Parameters.Add("@rcg", SqlDbType.UniqueIdentifier);
						param.Value = relConversationGroupHandle;
					}
					if (lifetime > TimeSpan.Zero)
					{
						query.Append(", LIFETIME = ");
						query.Append((long)lifetime.TotalSeconds);
						query.Append(' ');
					}

					param = cmd.Parameters.Add("@ch", SqlDbType.UniqueIdentifier);
					param.Direction = ParameterDirection.Output;
					param = cmd.Parameters.Add("@fs", SqlDbType.NVarChar, 255);
					param.Value = fromServiceName;
					param = cmd.Parameters.Add("@ts", SqlDbType.NVarChar, 255);
					param.Value = toServiceName;
					param = cmd.Parameters.Add("@cn", SqlDbType.NVarChar, 128);
					param.Value = contractName;

					cmd.CommandText = query.ToString(); ;

					cmd.ExecuteNonQuery();
					
					param = cmd.Parameters["@ch"] as SqlParameter;
					conversationHandle = (Guid)param.Value;

					conn.Close();

				    //commit if it worked.
					tranScope.Complete();
				}

				return conversationHandle;
			}
			catch
			{
				throw;
			}
			
		}
		
		/// <summary>
		/// Gets all the Conversations in a database as a Conversation Collection object.
		/// </summary>
		/// <param name="database">Name of the Database</param>
		/// <param name="conversationId">Conversation Id, Within a database it will return 2 conversations one on initiator side and other on target side.</param>
		/// <returns></returns>
		internal static ConversationCollection GetllAllConversations(Database database, string conversationId)
		{
			SqlConnection conn = ServiceBrokerManager.CreateDatabaseConnection(database);
			SqlCommand cmdConversations = conn.CreateCommand();
			cmdConversations.CommandText = String.Format(SqlStatements.CONVERSATION_BY_ID, conversationId);

			SqlDataReader drConversations = cmdConversations.ExecuteReader();
			ConversationCollection conversations = new ConversationCollection();
			while (drConversations.Read())
			{
				Conversation conv = new Conversation(
											  database
											, new Guid(Convert.ToString(drConversations[SqlColumnsConstant.ConversationHandle]))
											, new Guid(Convert.ToString(drConversations[SqlColumnsConstant.ConversationGroupHandle]))
											, Convert.ToDateTime(drConversations[SqlColumnsConstant.LifeTime])
											, drConversations.GetString(drConversations.GetOrdinal(SqlColumnsConstant.QueueName))
											, drConversations.GetString(drConversations.GetOrdinal(SqlColumnsConstant.ContractName))
											, drConversations.GetString(drConversations.GetOrdinal(SqlColumnsConstant.ServiceName))
											, drConversations.GetString(drConversations.GetOrdinal(SqlColumnsConstant.StateDescription))
											);

				conv.IsInitiator = drConversations.GetBoolean(drConversations.GetOrdinal(SqlColumnsConstant.IsInitiator));

				conversations.Add(conv);
			}
			drConversations.Close();

			for (int i = 0; i < conversations.Count; i++)
			{
				//Get messages from the corresponding queue
				Conversation conv = conversations.Item(i);
				if (conv != null)
				{
					SqlCommand cmdMessages = conn.CreateCommand();
					cmdMessages.CommandText = String.Format(String.Format(SqlStatements.MESSAGES_IN_QUEUE_BY_CONVERSATIONHANDLE, conv.QueueName, conv.ConversationHandle));

					SqlDataReader drMessages = cmdMessages.ExecuteReader();

					MessageCollection messages = new MessageCollection();

					while (drMessages.Read())
					{
						Message msg = new Message();
						msg.ContractName = drMessages.GetString(drMessages.GetOrdinal(SqlColumnsConstant.MessagesInQueue_ContractName));
						//msg.ConversationGroupId = new Guid(drMessages.GetString(drMessages.GetOrdinal(SqlColumnsConstant.MessagesInQueue_ConversationGroupId)));
						msg.ServiceName = drMessages.GetString(drMessages.GetOrdinal(SqlColumnsConstant.MessagesInQueue_ContractName));
						msg.SequenceNumber = drMessages.GetInt64(drMessages.GetOrdinal(SqlColumnsConstant.MessagesInQueue_MessageSequenceNumber));
						msg.Type = drMessages.GetString(drMessages.GetOrdinal(SqlColumnsConstant.MessagesInQueue_MessageTypeName));
						msg.Validation = drMessages.GetString(drMessages.GetOrdinal(SqlColumnsConstant.MessagesInQueue_Validation));
						msg.Status = drMessages.GetString(drMessages.GetOrdinal(SqlColumnsConstant.MessagesInQueue_Status));
						msg.QueueOrder = drMessages.GetInt64(drMessages.GetOrdinal(SqlColumnsConstant.MessagesInQueue_QueueOrder));

						if (drMessages.IsDBNull(drMessages.GetOrdinal(SqlColumnsConstant.MessagesInQueue_MessageBody)) == false)
						{
							SqlBytes sb = drMessages.GetSqlBytes(drMessages.GetOrdinal(SqlColumnsConstant.MessagesInQueue_MessageBody));
							msg.Body = sb.Stream;
						}
						else
							msg.Body = null;

						messages.Add(msg);
					}

					//Add messages collection to the conversation
					conversations.Item(i).Messages = messages;
					drMessages.Close();
				}
			}

			return conversations;
		}
		
		/// <summary>
		/// Gets all the Conversation Ids in the Database.
		/// </summary>
		/// <param name="database">Database name</param>
		/// <returns>Array of Conversation ids.</returns>
		internal static ArrayList GetAllConversationIds(Database database)
		{
			ArrayList conversationIds = new ArrayList();

			SqlConnection conn = ServiceBrokerManager.CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = SqlStatements.ALL_CONVERSATIONS_BY_ID;
			SqlDataReader dr = cmd.ExecuteReader();
			while (dr.Read())
			{
				conversationIds.Add((Guid)dr["CONVERSATION_ID"]);
			}

			dr.Close();

			return conversationIds;
		}
		
		/// <summary>
		/// Gets the corresponding Conversation Id for a given Conversation Handle
		/// </summary>
		/// <param name="database">Database Name where the Conversation is happening.</param>
		/// <param name="conversationHandle">Conversation Handle</param>
		/// <returns>Conversation Id</returns>
		internal static string GetConversationIdForHandle(Database database, string conversationHandle)
		{
			ArrayList conversationIds = new ArrayList();

			SqlConnection conn = ServiceBrokerManager.CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = String.Format(SqlStatements.CONVERSATION_ID_BY_HANDLE, conversationHandle);

			return Convert.ToString(cmd.ExecuteScalar());
		}
		
		/// <summary>
		/// Gets all the Conversation Handles in the Database.
		/// </summary>
		/// <param name="database">Database name</param>
		/// <returns>Array of conversation handles in the database.</returns>
		internal static ArrayList GetAllConversationHandles(Database database)
		{
			ArrayList conversationHandles = new ArrayList();

			SqlConnection conn = ServiceBrokerManager.CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = SqlStatements.ALL_CONVERSATIONS_BY_HANDLE;
			SqlDataReader dr = cmd.ExecuteReader();
			while (dr.Read())
			{
				conversationHandles.Add((Guid)dr["CONVERSATION_HANDLE"]);
			}

			dr.Close();

			return conversationHandles;
		}
		#endregion
	}
}
