//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Xml;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using System.Collections.Specialized;

using Microsoft.SqlServer.Management.Smo.Broker;
using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.MessageBox;

using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;
using Digitaldeposit.Ssbm.Resources;

namespace Digitaldeposit.Ssbm.Manager
{
	/// <summary>
	/// ServiceBrokerManager acts as the main interface point between the application and the SQL server.
	/// Any kind of communication like executing a query or getting some data out will be carried out through
	/// some of the methods present in this class.
	/// </summary>
	class ServiceBrokerManager
	{
		#region Fields
		XmlDocument _xmlDocument;
		#endregion

		#region Constructor
		public ServiceBrokerManager()
		{
			_xmlDocument = new XmlDocument();

		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Used to Authenticate the user
		/// </summary>
		/// <param name="loginType">Type of login ex SQL or Windows</param>
		/// <param name="userName">Username</param>
		/// <param name="password">Password</param>
		/// <param name="serverInstanceName">Server Instance Name</param>
		/// <returns></returns>
		public static object AuthenticateServer(MethodLoginType loginType, string userName, string password, string serverInstanceName)
		{
			ServerConnection conn = new ServerConnection();
			Server server = null;

			conn.ServerInstance = serverInstanceName;
			if (loginType == MethodLoginType.Windows)
				conn.LoginSecure = true;
			else
			{
				conn.LoginSecure = false;
				conn.Login = userName;
				conn.Password = password;
			}
			try
			{
				conn.Connect();

				return new Server(conn);
			}
			catch (ConnectionFailureException ex)
			{
				return ex;
			}
			finally
			{
				if (server != null && server.ConnectionContext.SqlConnectionObject.State == ConnectionState.Open)
					server.ConnectionContext.Disconnect();
			}
		}
		
		/// <summary>
		/// Used to construct the Xml data structure, which is used by the treeview in object explorer
		/// to construct the treeview.
		/// </summary>
		/// <param name="server">Server object</param>
		/// <returns>Either constructed Xml string or an Exception. Design to support Asynchrnous call.</returns>
		public object ConstructObjectExplorerXml(Server server)
		{
			try
			{
				string serverName = server.Name;

				XmlNode nServers = _xmlDocument.AppendChild(CreateNode(SsbmObjectType.Server.ToString("g"), ObjectExplorer.Treeview_Servers, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Server, string.Empty, String.Empty, NodeType.Parent, TreeNodeExpandRequired.No));
				
				//Server Node
				XmlNode nServer = nServers.AppendChild(CreateNode(SsbmObjectType.Server.ToString("g"), serverName, String.Empty, ProjectImagesConstants.RegisteredServers, (int)SsbmObjectType.Server, serverName, String.Empty, NodeType.Child, TreeNodeExpandRequired.No));

				//Databases Nodes.
				XmlNode nDatabases = nServer.AppendChild(CreateNode(SsbmObjectType.Database.ToString("g"), ObjectExplorer.Treeview_Databases, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Database, serverName, String.Empty, NodeType.Parent, TreeNodeExpandRequired.No));
				
				//Security Node
				XmlNode nSecurity = nServer.AppendChild(CreateNode(XmlNodeAttributeConstants.SECURITY, ObjectExplorer.Treeview_Security, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.None, String.Empty, String.Empty, NodeType.Parent, TreeNodeExpandRequired.No));
				XmlNode nLogin = nSecurity.AppendChild(CreateNode(SsbmObjectType.Login.ToString("g"), ObjectExplorer.Treeview_Login, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Login, server.Name, String.Empty, NodeType.Parent, TreeNodeExpandRequired.Yes));
				XmlNode nEndPoint = nSecurity.AppendChild(CreateNode(SsbmObjectType.EndPoint.ToString("g"), ObjectExplorer.Treeview_EndPoint, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.EndPoint, serverName, String.Empty, NodeType.Parent, TreeNodeExpandRequired.Yes));

				//Monitoring Node
				XmlNode nMonitoring = nServer.AppendChild(CreateNode(XmlNodeAttributeConstants.MONITORING, ObjectExplorer.Treeview_MonitoringServiceBroker, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.None, String.Empty, String.Empty, NodeType.Parent, TreeNodeExpandRequired.No));
				//nMonitoring.AppendChild(CreateNode(XmlNodeAttributeConstants.MESSAGEFINDER, ObjectExplorer.Treeview_MessageFinder, String.Empty, ProjectImagesConstants.MessageFinder, (int)SsbmObjectType.None, String.Empty, String.Empty, NodeType.Child, TreeNodeExpandRequired.No));
				nMonitoring.AppendChild(CreateNode(XmlNodeAttributeConstants.ACTIVATED_TASKS, ObjectExplorer.Treeview_ActivatedTasks, String.Empty, ProjectImagesConstants.ActivatedTasks, (int)SsbmObjectType.ActivatedTasks, serverName, String.Empty, NodeType.Child, TreeNodeExpandRequired.No));
				nMonitoring.AppendChild(CreateNode(XmlNodeAttributeConstants.BROKER_CONNECTIONS, ObjectExplorer.Treeview_BrokerConnections, String.Empty, ProjectImagesConstants.BrokerConnections, (int)SsbmObjectType.BrokerConnections, serverName, String.Empty, NodeType.Child, TreeNodeExpandRequired.No));
				nMonitoring.AppendChild(CreateNode(XmlNodeAttributeConstants.FORWARDED_MESSAGES, ObjectExplorer.Treeview_ForwardedMessages, String.Empty, ProjectImagesConstants.ForwardMessage, (int)SsbmObjectType.ForwardedMessages, serverName, String.Empty, NodeType.Child, TreeNodeExpandRequired.No));
				nMonitoring.AppendChild(CreateNode(XmlNodeAttributeConstants.QUEUE_MONITORS, ObjectExplorer.Treeview_QueueMonitors, String.Empty, ProjectImagesConstants.QueueMonitors, (int)SsbmObjectType.QueueMonitors, serverName, serverName, NodeType.Child, TreeNodeExpandRequired.No));

				string databaseName = string.Empty;
				
				//Lock the collection
				//lock (server.Databases.SyncRoot)
				{
					foreach (Database db in server.Databases)
					{
						databaseName = db.Name;

						//Database Node
						XmlNode nDb = nDatabases.AppendChild(CreateNode(SsbmObjectType.Database.ToString("g"), databaseName, string.Empty, ProjectImagesConstants.Database, (int)SsbmObjectType.Database, serverName, databaseName, NodeType.Child, TreeNodeExpandRequired.No));

						//Create Top Level container nodes
						XmlNode nArtifacts = nDb.AppendChild(CreateNode(XmlNodeAttributeConstants.ARTIFACTS, ObjectExplorer.Treeview_Artifacts, string.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.None, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.No));
						XmlNode nDependency = nDb.AppendChild(CreateNode(XmlNodeAttributeConstants.DEPENDENCY, ObjectExplorer.Treeview_Dependency, string.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.None, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.No));
						XmlNode nRuntime = nDb.AppendChild(CreateNode(XmlNodeAttributeConstants.RUNTIME, ObjectExplorer.Treeview_Runtime, string.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.None, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.No));
						XmlNode nDBSecurtiy = nDb.AppendChild(CreateNode(XmlNodeAttributeConstants.SECURITY, ObjectExplorer.Treeview_Security, string.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.None, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.No));

						//SSB Artifacts Node
						nArtifacts.AppendChild(CreateNode(SsbmObjectType.MessageType.ToString("g"), ObjectExplorer.Treeview_MessageTypes, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.MessageType, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));
						nArtifacts.AppendChild(CreateNode(SsbmObjectType.Contract.ToString("g"), ObjectExplorer.Treeview_Contracts, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Contract, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));
						nArtifacts.AppendChild(CreateNode(SsbmObjectType.Queue.ToString("g"), ObjectExplorer.Treeview_Queues, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Queue, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));
						nArtifacts.AppendChild(CreateNode(SsbmObjectType.Service.ToString("g"), ObjectExplorer.Treeview_Services, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Service, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));
						nArtifacts.AppendChild(CreateNode(SsbmObjectType.Route.ToString("g"), ObjectExplorer.Treeview_Routes, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Route, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));
						nArtifacts.AppendChild(CreateNode(SsbmObjectType.RemoteBinding.ToString("g"), ObjectExplorer.Treeview_RemoteServiceBindings, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.RemoteBinding, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));
		
						//Dependency
						nDependency.AppendChild(CreateNode(SsbmObjectType.SchemaCollection.ToString("g"), ObjectExplorer.Treeview_SchemaCollection, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.SchemaCollection, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));
						nDependency.AppendChild(CreateNode(SsbmObjectType.ActivationSproc.ToString("g"), ObjectExplorer.Treeview_ActivationStoredProcedures, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.ActivationSproc, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));

						//Runtime
						nRuntime.AppendChild(CreateNode(SsbmObjectType.Conversation.ToString("g"), ObjectExplorer.Treeview_Conversations, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Conversation, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));
						nRuntime.AppendChild(CreateNode(XmlNodeAttributeConstants.TRANSMISSIONQ, ObjectExplorer.Treeview_TransmissionQueue, String.Empty, ProjectImagesConstants.Queue, (int)SsbmObjectType.TransmissionQ, serverName, databaseName, NodeType.Child, TreeNodeExpandRequired.No));

						//Security
						nDBSecurtiy.AppendChild(CreateNode(SsbmObjectType.User.ToString("g"), ObjectExplorer.Treeview_User, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.User, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));
						nDBSecurtiy.AppendChild(CreateNode(SsbmObjectType.Certificate.ToString("g"), ObjectExplorer.Treeview_Certificate, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Certificate, serverName, databaseName, NodeType.Parent, TreeNodeExpandRequired.Yes));
			
					}
				}

				//For testing
				//_xmlDocument.Save("c:\\test.xml");
				return _xmlDocument.OuterXml;
			}
			catch(Exception ex)
			{
				return ex;
			}
		}

		/// <summary>
		/// Used to construct the Xml data structure, which is used by the treeview in Route, Remote Scripting and Conversation
		/// Object maintenance screens.
		/// </summary>
		/// <param name="server">Server Collection array</param>
		/// <returns>Either constructed Xml string or an Exception. Design to support Asynchrnous call.</returns>
		public object ConstructRemoteServicesXml(Server[] server)
		{
			try
			{
				XmlNode nServers = _xmlDocument.AppendChild(CreateNode(SsbmObjectType.Server.ToString("g"), ObjectExplorer.Treeview_Servers, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Server, string.Empty, String.Empty, NodeType.Parent, TreeNodeExpandRequired.No));

				for (int i = 0; i < server.Length; i++)
				{
					string serverName = server[i].Name;

					
					//Server Node
					XmlNode nServer = nServers.AppendChild(CreateNode(SsbmObjectType.Server.ToString("g"), serverName, String.Empty, ProjectImagesConstants.RegisteredServers, (int)SsbmObjectType.Server, serverName, String.Empty, NodeType.Child, TreeNodeExpandRequired.No));

					//Databases Nodes.
					XmlNode nDatabases = nServer.AppendChild(CreateNode(SsbmObjectType.Database.ToString("g"), ObjectExplorer.Treeview_Databases, String.Empty, ProjectImagesConstants.Folder, (int)SsbmObjectType.Database, serverName, String.Empty, NodeType.Parent, TreeNodeExpandRequired.No));


					string databaseName = string.Empty;
					foreach (Database db in server[i].Databases)
					{
						ServiceBroker sb = db.ServiceBroker;
						databaseName = db.Name;
						//Database Node
						XmlNode nDb = nDatabases.AppendChild(CreateNode(SsbmObjectType.Database.ToString("g"), databaseName, string.Empty, ProjectImagesConstants.Database, (int)SsbmObjectType.Database, serverName, databaseName, NodeType.Child, TreeNodeExpandRequired.No));

						foreach (BrokerService bs in sb.Services)
						{
							nDb.AppendChild(CreateNode(SsbmObjectType.Service.ToString("g"),bs.Name,string.Empty,ProjectImagesConstants.Service,(int)SsbmObjectType.Service,serverName,databaseName,NodeType.Child,TreeNodeExpandRequired.NotRequired));
						}
					}
				}

				//For testing
				_xmlDocument.Save("c:\\service.xml");
				return _xmlDocument.OuterXml;
			}
			catch (Exception ex)
			{
				return ex;
			}
		}
		
		/// <summary>
		/// Used to create every single node in the Xml.
		/// </summary>
		/// <param name="name">Name of the Node</param>
		/// <param name="displayName">Display Name attribute value</param>
		/// <param name="owner">Owner attribute value</param>
		/// <param name="imageKey">Image key attribute value</param>
		/// <param name="ssbType">SsbmObjectType attribute value</param>
		/// <param name="serverName">Server name attribute value</param>
		/// <param name="databaseName">Database name attribute value</param>
		/// <param name="nodeType">Node type, could be parent or child depending on their position.</param>
		/// <param name="tvExpandRequired">Expand required could be Yes, No, Notrequired based on whether we need to expansion at runtime or its already done.</param>
		/// <returns></returns>
		public XmlNode CreateNode(string name, string displayName, string owner, string imageKey, int ssbType, string serverName, string databaseName, NodeType nodeType, TreeNodeExpandRequired tvExpandRequired)
		{
			try
			{
				XmlNode e = _xmlDocument.CreateElement(name);

				XmlAttribute aDN = _xmlDocument.CreateAttribute(XmlNodeAttributeConstants.DISPLAY_NAME);
				aDN.InnerText = displayName;

				XmlAttribute aOwner = _xmlDocument.CreateAttribute(XmlNodeAttributeConstants.OWNER);
				aOwner.InnerText = owner;

				XmlAttribute aImageIndex = _xmlDocument.CreateAttribute(XmlNodeAttributeConstants.IMAGEKEY);
				aImageIndex.InnerText = imageKey;

				XmlAttribute aSSBType = _xmlDocument.CreateAttribute(XmlNodeAttributeConstants.SSBTYPE);
				aSSBType.InnerText = Convert.ToString(ssbType);

				XmlAttribute aServer = _xmlDocument.CreateAttribute(XmlNodeAttributeConstants.SERVER);
				aServer.InnerText = serverName;

				XmlAttribute aDatabase = _xmlDocument.CreateAttribute(XmlNodeAttributeConstants.DATABASE);
				aDatabase.InnerText = databaseName;

				XmlAttribute aNodeType = _xmlDocument.CreateAttribute(XmlNodeAttributeConstants.NODETYPE);
				aNodeType.InnerText = Convert.ToString((int)nodeType);

				XmlAttribute aGuid = _xmlDocument.CreateAttribute(XmlNodeAttributeConstants.GUID);
				aGuid.InnerText = Convert.ToString(Guid.NewGuid());

				XmlAttribute aExpandRequired = _xmlDocument.CreateAttribute(XmlNodeAttributeConstants.EXPAND_REQUIRED);
				aExpandRequired.InnerText = Convert.ToString((int)tvExpandRequired); ;

				e.Attributes.Append(aDN);
				e.Attributes.Append(aImageIndex);
				e.Attributes.Append(aSSBType);
				e.Attributes.Append(aServer);
				e.Attributes.Append(aDatabase);
				e.Attributes.Append(aOwner);
				e.Attributes.Append(aNodeType);
				e.Attributes.Append(aExpandRequired);
				e.Attributes.Append(aGuid);
				return e;
			}
			catch
			{
				throw;
			}
		}
		#endregion

		#region Internal Static Methods
		/// <summary>
		/// Used to create a ADO Connection object from Database object.
		/// </summary>
		/// <param name="db">Database object</param>
		/// <returns>Sql Connection</returns>
		internal static SqlConnection CreateDatabaseConnection(Database db)
		{
			SqlConnection conn = new SqlConnection();
			conn.ConnectionString = db.Parent.ConnectionContext.SqlConnectionObject.ConnectionString;
			conn.Open();

			if (conn.Database != db.Name)
				conn.ChangeDatabase(db.Name);
			return conn;
		}
		
		/// <summary>
		/// Returns whether Service broker is enabled or disabled for the corresponding Database.
		/// </summary>
		/// <param name="database">Database object</param>
		/// <returns>bool value based on the status</returns>
		internal static bool GetServiceBrokerStatus(Database database)
		{
			SqlConnection conn = CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = String.Format(SqlStatements.SERVICE_BROKER_STATUS_BY_DATABASE, database.Name);
			return Convert.ToBoolean(cmd.ExecuteScalar());
		}

		/// <summary>
		/// Get same basic properties of the database like, wheter service broker is enabled, master key is setup etc.
		/// </summary>
		/// <param name="database">Database Object</param>
		/// <returns>Dataset containing values.</returns>
		internal static DataSet GetDataBaseProperties(Database database)
		{
			Hashtable dbProps = new Hashtable();

			SqlConnection conn = CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = String.Format(SqlStatements.DATABASE_PROPERTIES, database.Name);
			SqlDataAdapter da = new SqlDataAdapter(cmd);
			DataSet ds = new DataSet();
			da.Fill(ds, ServiceBrokerConstants.DatabaseProperties);
			return ds;
		}

		/// <summary>
		/// Used to get the messages from corresonding queue.
		/// </summary>
		/// <param name="database">Database object</param>
		/// <param name="queueName">Name of the queue</param>
		/// <returns>Dataset containing values</returns>
		internal static DataSet GetMessagesFromQueue(Database database, string queueName)
		{
			SqlConnection conn =  CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = String.Format(SqlStatements.MESSAGES_IN_QUEUE, queueName);
			SqlDataAdapter da = new SqlDataAdapter(cmd);
			DataSet ds = new DataSet();
			da.Fill(ds, queueName);
			return ds;
		}

		/// <summary>
		/// Used to get the status of the queue.
		/// </summary>
		/// <param name="database">Database object</param>
		/// <param name="queueName">Queue Name</param>
		/// <returns>Dataset containing values</returns>
		internal static DataSet GetQueueStatus(Database database, string queueName)
		{
			SqlConnection conn = CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = String.Format(SqlStatements.QUEUE_STATUS_BY_QUEUENAME,queueName);
			SqlDataAdapter da = new SqlDataAdapter(cmd);
			DataSet ds = new DataSet();
			da.Fill(ds, queueName);
			return ds;
		}
		
		/// <summary>
		/// Used to get the status of all the queues in the database
		/// </summary>
		/// <param name="database">Database object</param>
		/// <returns>Dataset containing values</returns>
		internal static DataSet GetQueueStatus(Database database)
		{
			SqlConnection conn = CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = SqlStatements.ALL_QUEUE_STATUS;
			SqlDataAdapter da = new SqlDataAdapter(cmd);
			DataSet ds = new DataSet();
			da.Fill(ds, ServiceBrokerConstants.QueueStatus);
			return ds;
		}

		/// <summary>
		/// Used get all the messages in the transmission queue.
		/// </summary>
		/// <param name="database">Database object</param>
		/// <returns>Dataset containing values</returns>
		internal static DataSet GetMessagesFromTransmission(Database database)
		{
			SqlConnection conn = CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = SqlStatements.MESSAGES_IN_TRANSMISSION_QUEUE;
			SqlDataAdapter da = new SqlDataAdapter(cmd);
			DataSet ds = new DataSet();
			da.Fill(ds, ServiceBrokerConstants.TransmissionQ);
			return ds;
		}
	
		/// <summary>
		/// Used to enable service broker in the database.
		/// </summary>
		/// <param name="database">Database object</param>
		internal static void EnableServiceBroker(Database database)
		{
			SqlConnection conn = CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = String.Format(SqlStatements.ENABLE_SERVICE_BROKER, database.Name);
			cmd.ExecuteNonQuery();
		}

		/// <summary>
		/// Used to Disable Service broker in the database.
		/// </summary>
		/// <param name="database">Database object.</param>
		internal static void DisableServiceBroker(Database database)
		{
			SqlConnection conn = CreateDatabaseConnection(database);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = String.Format(SqlStatements.DISABLE_SERVICE_BROKER, database.Name);
			cmd.ExecuteNonQuery();
		}

		/// <summary>
		///	Used to display details of stored procedure activated by Service Broker.
		/// </summary>
		/// <param name="server">Server object</param>
		/// <returns>Dataset containing values</returns>
		internal static DataSet GetActivatedTasks(Server server)
		{
			Database db = new Database(server, "master");
			SqlConnection conn = CreateDatabaseConnection(db);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = SqlStatements.ACTIVATED_TASKS;
			SqlDataAdapter da = new SqlDataAdapter(cmd);
			DataSet ds = new DataSet();
			da.Fill(ds, ServiceBrokerConstants.ActivatedTask);
			return ds;
		}

		/// <summary>
		/// Used to display details of Service Broker network connection.
		/// </summary>
		/// <param name="server">Server object</param>
		/// <returns>Dataset containing values</returns>
		internal static DataSet GetBrokerConnections(Server server)
		{
			Database db = new Database(server, "master");
			SqlConnection conn = CreateDatabaseConnection(db);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = SqlStatements.BROKER_CONNECTIONS;
			SqlDataAdapter da = new SqlDataAdapter(cmd);
			DataSet ds = new DataSet();
			da.Fill(ds, ServiceBrokerConstants.BrokerConnections);
			return ds;
		}

		/// <summary>
		/// Used to display details of Service Broker message that the SQL Server instance is in the process of forwarding.
		/// </summary>
		/// <param name="server">Server object</param>
		/// <returns>Dataset containing values</returns>
		internal static DataSet GetForwardedMessages(Server server)
		{
			Database db = new Database(server, "master");
			SqlConnection conn = CreateDatabaseConnection(db);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = SqlStatements.FORWARDED_MESSAGES;
			SqlDataAdapter da = new SqlDataAdapter(cmd);
			DataSet ds = new DataSet();
			da.Fill(ds, ServiceBrokerConstants.ForwardedMessages);
			return ds;
		}

		/// <summary>
		/// Used to display details of queue monitor in the instance. A queue monitor manages activation for a queue.
		/// </summary>
		/// <param name="server">Server object</param>
		/// <returns>Dataset containing values</returns>
		internal static DataSet GetQueueMonitors(Server server)
		{
			Database db = new Database(server, "master");
			SqlConnection conn = CreateDatabaseConnection(db);
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = SqlStatements.QUEUE_MONITORS;
			SqlDataAdapter da = new SqlDataAdapter(cmd);
			DataSet ds = new DataSet();
			da.Fill(ds, ServiceBrokerConstants.QueueMonitors);
			return ds;
		}
		#endregion
	}
}
