//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

using Microsoft.SqlServer.Management.Smo;

using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;

namespace Digitaldeposit.Ssbm.ObjectMaintenance
{
	/// <summary>
	/// Class containing Event Data for different operations like Create, Delete, Alter etc.
	/// </summary>
	public class SsbmEventArgs : EventArgs
	{
		#region Fields
		XmlNode _info;
		SsbMaintenanceOperation _operation;
		SsbmObjectType _ssbType;
		Server _server;
		Database _database;
		#endregion

		#region  Properties
		/// <summary>
		/// XmlNode object containing all the information
		/// Ex:  <MessageType DN="Message Types" ImageKey="Folder" SSBType="0" Server="DDL" Database="AdventureWorks" Owner="" NodeType="0" ExpandRequired="0" Guid="1924f68a-8437-4d68-8d46-0e5d7d1c7725" />
		/// </summary>
    	public XmlNode Info
		{ 
			get{return _info;}
		}

		/// <summary>
		/// Type of operation, Ex: Create, Alter, Delete etc.
		/// </summary>
		public SsbMaintenanceOperation Operation
		{
			get{return _operation;}
		}

		/// <summary>
		/// Type of SsbmObject, Ex: MessageType, Queue, Contract etc
		/// </summary>
		public SsbmObjectType SsbmObjectType
		{ 
			get { return _ssbType; } 
		}

		/// <summary>
		/// SQL Server instance object
		/// </summary>
		public Server Server
		{ 
			get { return _server; } 
		}

		/// <summary>
		/// Database object
		/// </summary>
		public Database Database
		{ 
			get { return _database; }
		}
		#endregion

		#region Constructor
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="info">XmlNode object containing all the information</param>
		/// <param name="operation">Type of operation, Ex: Create, Alter, Delete etc.</param>
		/// <param name="ssbmObjectType">Type of SsbmObject, Ex: MessageType, Queue, Contract etc</param>
		/// <param name="server">SQL Server instance object</param>
		/// <param name="database">Database object</param>
		public SsbmEventArgs(XmlNode info, SsbMaintenanceOperation operation, SsbmObjectType ssbmObjectType, Server server, Database database)
		{
			_info = info;
			_operation = operation;
			_ssbType = ssbmObjectType;
			_server = server;
			_database = database;
		}
		#endregion
	}
}
