//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Collections.Specialized;
using System.Xml;
using System.Windows.Forms;

using DevComponents.DotNetBar;

using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Smo.Broker;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.MessageBox;

using Digitaldeposit.Ssbm.ObjectMaintenance.Forms;
using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;
using Digitaldeposit.Ssbm.CustomControls;

namespace Digitaldeposit.Ssbm.ObjectMaintenance.UserControls
{
	/// <summary>
	/// Takes care of all the UI related operation to create a SQL Server instance Endpoint object.
	/// </summary>
	public partial class EndPointMaintenance : UserControl
	{
		#region Constructor
		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		public EndPointMaintenance(Digitaldeposit.Ssbm.ObjectMaintenance.Forms.CommonForm commonForm)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "ContractMaintenance"));

			InitializeComponent();
			Setup(commonForm);
		}
		#endregion

		#region Private Methods
		private void Setup(Digitaldeposit.Ssbm.ObjectMaintenance.Forms.CommonForm commonForm)
		{
			try
			{
				//1. Authorization
				cmbAuthorization.ImageList = commonForm.ImageList;
				cmbAuthorization.Items.Clear();
				lock (commonForm.Server.Logins.SyncRoot)
				{
					foreach (Login login in commonForm.Server.Logins)
						cmbAuthorization.Items.Add(new ComboBoxWithImagesItem(login.Name, commonForm.ImageList.Images.IndexOfKey(ProjectImagesConstants.Login)));
				}

				//2. Authentication
				cmbAuthentication.DataSource = Enum.GetValues(typeof(EndpointAuthenticationOrder));
				cmbAuthentication.SelectedIndex = (int)EndpointAuthenticationOrder.Kerberos;
       
				//3. Certificate
				//cmbCertificate.ImageList = commonForm.ImageList;
				//cmbCertificate.Items.Clear();
				//lock (commonForm.Server.)
				//{
				//    foreach (Login login in commonForm.Server.Logins)
				//        cmbCertificate.Items.Add(new ComboBoxWithImagesItem(login.Name, commonForm.ImageList.Images.IndexOfKey(ProjectImagesConstants.Login)));
				//}

				//4. Encryption
				cmbEncryption.DataSource = Enum.GetValues(typeof(EndpointEncryption));
				cmbEncryption.SelectedIndex = (int)EndpointEncryption.Required;
       
				//5. Algorithm
				cmbAlgorithm.DataSource = Enum.GetValues(typeof(EndpointEncryptionAlgorithm));
				cmbAlgorithm.SelectedIndex = (int)EndpointEncryptionAlgorithm.RC4;

				//6. Port, IP
				txtListenerIP.Text = "ALL";
				txtListenerPort.Text = "4022";

				//Setup Accessibility
				txtMsgForwasdSize.Enabled = false;
				cmbCertificate.Enabled = false;
       	

				if (commonForm.Operation  == SsbMaintenanceOperation.Alter)
				{
					Endpoint endPoint = null;
					lock (commonForm.Server.Endpoints.SyncRoot)
					{
						endPoint = commonForm.Server.Endpoints[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
					}
					SetupForEditOperation(endPoint);
				}
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		private void SetupForEditOperation(Endpoint endPoint)
		{
			try
			{
				//Name
				txtName.Text = endPoint.Name;
				txtName.Enabled = false; //Can't modify the name.

				//Authorization
				//if (mt.Owner.Length != 0)
				//    cmbAuthorization.SelectedItem = mt.Owner;

				}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		private bool ValidateControls()
		{
			bool result = true;

			if (txtName.Text.Length == 0)
			{
				result = false;
				throw new InvalidArgumentException(String.Format(Resources.Exceptions.ObjectMaintenance_InvalidName, "Endpoint"));
			}
			return result;
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Called from form to populate Endpoint object with corresponding UI data.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		/// <returns>A populated Endpoint object which can then undergo any SQL Operation like Create() or Alter()</returns>
		public Endpoint Populate(Digitaldeposit.Ssbm.ObjectMaintenance.Forms.CommonForm commonForm)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "Endpoint Maintenance"));

			Endpoint endPoint = null;
			if (commonForm.Operation != SsbMaintenanceOperation.Alter)
			{
				endPoint = new Endpoint();
				endPoint.Name = txtName.Text;
				endPoint.Parent = commonForm.Server;
				endPoint.ProtocolType = ProtocolType.Tcp;
				endPoint.EndpointType = EndpointType.ServiceBroker;
			}
			else
			{
				lock (commonForm.Server.Endpoints.SyncRoot)
				{
					endPoint = commonForm.Server.Endpoints[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
				}
			}

			//Port, IP
			endPoint.Protocol.Tcp.ListenerPort = int.Parse(txtListenerPort.Text);

			if (txtListenerIP.Text == "ALL")
				endPoint.Protocol.Tcp.ListenerIPAddress = System.Net.IPAddress.Any;
			else
			{
				endPoint.Protocol.Tcp.ListenerIPAddress = System.Net.IPAddress.Parse(txtListenerIP.Text);
			}

			//Authentication
			endPoint.Payload.ServiceBroker.EndpointAuthenticationOrder = (EndpointAuthenticationOrder)cmbAuthentication.SelectedItem;

			//Certificate //TODO: SMO not supporting this property at the moment.
			//if (cmbCertificate.SelectedIndex != -1)
			//    endPoint.Payload.ServiceBroker. = ep_cboCert.Text;

			//Encryption
			endPoint.Payload.ServiceBroker.EndpointEncryption = (EndpointEncryption)cmbEncryption.SelectedItem;

			//Algorithm
			if (cmbAlgorithm.SelectedIndex != -1)
				endPoint.Payload.ServiceBroker.EndpointEncryptionAlgorithm = (EndpointEncryptionAlgorithm)cmbAlgorithm.SelectedItem;

			//State
			//switch ((EndpointState)cmbState.SelectedIndex)
			//{
			//    case EndpointState.Disabled: endPoint.Disable(); break;
			//    case EndpointState.Started: endPoint.Start(); break;
			//    case EndpointState.Stopped: endPoint.Stop(); break;
			//}

			//Message Forwarding
			endPoint.Payload.ServiceBroker.IsMessageForwardingEnabled = chkMsgForwarding.Checked;

			if (chkMsgForwarding.Checked)
				endPoint.Payload.ServiceBroker.MessageForwardingSize = int.Parse(txtMsgForwasdSize.Text);

			if (ValidateControls())
			{
				return endPoint;
			}
			return null;
		}
		#endregion
	}
}
