//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Collections.Specialized;
using System.Xml;
using System.Windows.Forms;

using DevComponents.DotNetBar;

using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Smo.Broker;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.MessageBox;

using Digitaldeposit.Ssbm.ObjectMaintenance.Forms;
using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;

namespace Digitaldeposit.Ssbm.ObjectMaintenance.UserControls
{
	/// <summary>
	/// Takes care of all the UI related operation to create a SQL Server Login object.
	/// </summary>
	public partial class LoginMaintenance : UserControl
	{
		#region Constructor
		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		public LoginMaintenance(Digitaldeposit.Ssbm.ObjectMaintenance.Forms.CommonForm commonForm)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "ContractMaintenance"));

			InitializeComponent();
			Setup(commonForm);
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Called from form to populate Login object with corresponding UI data.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		/// <param name="additionalReturnValues">Some of the values cannot be set to the return Certificate object, so add them to this hashtable.</param>
		/// <returns>A populated Login object which can then undergo any SQL Operation like Create() or Alter()</returns>
		public Login Populate(Digitaldeposit.Ssbm.ObjectMaintenance.Forms.CommonForm commonForm, ref Hashtable additionalReturnValues)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "Login Maintenance"));

			if(additionalReturnValues == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "additionalReturnValues", "Login Maintenance"));


			Login login;
			string loginName = string.Empty;

			//Do things based on operation either create or alter.
			if (commonForm.Operation == SsbMaintenanceOperation.Alter)
			{
				login = commonForm.Server.Logins[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
			}
			else
			{
				login = new Login();
				login.Parent = commonForm.Server;
				

				if (rbWindowsAuth.Checked)
				{
					login.Name = txtWindowsAuthLogin.Text;
					loginName = txtWindowsAuthLogin.Text;
					login.LoginType = LoginType.WindowsUser;
				}
				else if (rbSQLAuth.Checked)
				{
					login.Name = txtSQLAuthLogin.Text;
					loginName = txtSQLAuthLogin.Text;
					login.LoginType = LoginType.SqlLogin;
				}
			}

			if (rbSQLAuth.Checked)
			{
				login.PasswordExpirationEnabled = chkEnforcePasswordExpiration.Checked;
				login.PasswordPolicyEnforced = chkEnforcePasswordPolicy.Checked;
			}

			//Password cannot be set on login object, so pass it as additional value.
			additionalReturnValues.Clear();
			additionalReturnValues.Add(ServiceBrokerConstants.LoginPassword, txtSQLAuthPassword.Text);

			if (ValidateControls())
			{
				return login;
			}

			return null;
		}
		#endregion

		#region Event Handlers
		private void Authentication_ModeChanged(object sender, EventArgs e)
		{
			txtWindowsAuthLogin.Enabled = rbWindowsAuth.Checked;
			txtSQLAuthLogin.Enabled = rbSQLAuth.Checked;
			txtSQLAuthPassword.Enabled = rbSQLAuth.Checked;
			txtSQLAuthConfirmPassword.Enabled = rbSQLAuth.Checked;
			chkEnforcePasswordExpiration.Enabled = rbSQLAuth.Checked;
			chkEnforcePasswordPolicy.Enabled = rbSQLAuth.Checked;
		}
		#endregion

		#region Private Methods
		private void Setup(Digitaldeposit.Ssbm.ObjectMaintenance.Forms.CommonForm commonForm)
		{
			try
			{
				if (commonForm.Operation  == SsbMaintenanceOperation.Alter)
				{
					Login login = commonForm.Server.Logins[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
					SetupForEditOperation(login);
				}
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		private void SetupForEditOperation(Login login)
		{
			try
			{
				chkEnforcePasswordExpiration.Checked = login.PasswordExpirationEnabled;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				chkEnforcePasswordExpiration.Checked = false;
			}


			try
			{
				chkEnforcePasswordPolicy.Checked = login.PasswordPolicyEnforced;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				chkEnforcePasswordPolicy.Checked = false;
			}

			//Name;
			try
			{
				if (login.LoginType == LoginType.WindowsUser)
				{
					rbWindowsAuth.Checked = true;
					txtWindowsAuthLogin.Text = login.Name;
				}
				else if (login.LoginType == LoginType.SqlLogin)
				{
					rbSQLAuth.Checked = true;
					txtSQLAuthLogin.Text = login.Name;
				}
			}
			catch (PropertyCannotBeRetrievedException)
			{
				//
			}
			txtSQLAuthLogin.Enabled = false;
			txtWindowsAuthLogin.Enabled = false;
		
		}
		private bool ValidateControls()
		{
			bool result = true;

			if (rbSQLAuth.Checked)
			{
				if (txtSQLAuthLogin.Text.Length == 0)
				{
					result = false;
					throw new InvalidArgumentException(String.Format(Resources.Exceptions.ObjectMaintenance_InvalidName, "Login"));
				}

				if (txtSQLAuthPassword.Text != txtSQLAuthConfirmPassword.Text)
				{
					result = false;
					throw new InvalidArgumentException(Resources.Exceptions.ObjectMaintenance_InvalidConfirmPassword);
				}
			}

			if (rbWindowsAuth.Checked && txtWindowsAuthLogin.Text.Length == 0)
			{
				if (txtWindowsAuthLogin.Text.Length == 0)
				{
					result = false;
					throw new InvalidArgumentException(String.Format(Resources.Exceptions.ObjectMaintenance_InvalidName, "Login"));
				}
			}
		
			return result;
		}
		#endregion
	}
}
