//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;

using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Smo.Broker;

using Digitaldeposit.Ssbm.Manager;
using Digitaldeposit.Ssbm.Constants;

namespace Digitaldeposit.Ssbm.PropertyMaintenance
{
	static class SsbmObjectTypeProperties
	{
		#region Internal Static Methods
		internal static PropertyTable GetServerProperties(Server server)
		{
			if (server == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "server"));

			PropertyTable propertyTable = new PropertyTable();


			//****************** AuthenticationMethod ******************
			PropertySpec psAuthenticationMethod = null;
			try
			{
				psAuthenticationMethod = new PropertySpec(Resources.Properties.Property_Server_AuthenticationMethod_Text
														, typeof(string)
														, Resources.Properties.Property_Server_AuthenticationMethod_Category
														, Resources.Properties.Property_Server_AuthenticationMethod_Description);

				if (server.ConnectionContext.LoginSecure)
					propertyTable[Resources.Properties.Property_Server_AuthenticationMethod_Text] = Digitaldeposit.Ssbm.Common.Enums.SqlServerAuthenticationMode.Windows.ToString("g");
				else
					propertyTable[Resources.Properties.Property_Server_AuthenticationMethod_Text] = Digitaldeposit.Ssbm.Common.Enums.SqlServerAuthenticationMode.Mixed.ToString("g");

				psAuthenticationMethod.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psAuthenticationMethod.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psAuthenticationMethod);


			//****************** ProductName ******************
			PropertySpec psProductName = null;
			try
			{
				psProductName = new PropertySpec(Resources.Properties.Property_Server_ProductName_Text
														, typeof(string)
														, Resources.Properties.Property_Server_ProductName_Category
														, Resources.Properties.Property_Server_ProductName_Description);

				propertyTable[Resources.Properties.Property_Server_ProductName_Text] = server.Information.Product;

				psProductName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProductName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProductName);

			//****************** ProductVersion ******************
			PropertySpec psProductVersion = null;
			try
			{
				psProductVersion = new PropertySpec(Resources.Properties.Property_Server_ProductVersion_Text
														, typeof(string)
														, Resources.Properties.Property_Server_ProductVersion_Category
														, Resources.Properties.Property_Server_ProductVersion_Description);

				propertyTable[Resources.Properties.Property_Server_ProductVersion_Text] = server.Information.Version.ToString();

				psProductVersion.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProductVersion.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProductVersion);

			//****************** ServerName ******************
			PropertySpec psServerName = null;
			try
			{
				psServerName = new PropertySpec(Resources.Properties.Property_Server_ServerName_Text
														, typeof(string)
														, Resources.Properties.Property_Server_ServerName_Category
														, Resources.Properties.Property_Server_ServerName_Description);

				propertyTable[Resources.Properties.Property_Server_ServerName_Text] = server.Name;
				psServerName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psServerName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psServerName);

			//****************** InstanceName ******************
			PropertySpec psInstanceName = null;
			try
			{
				psInstanceName = new PropertySpec(Resources.Properties.Property_Server_InstanceName_Text
														, typeof(string)
														, Resources.Properties.Property_Server_InstanceName_Category
														, Resources.Properties.Property_Server_InstanceName_Description);

				propertyTable[Resources.Properties.Property_Server_InstanceName_Text] = server.InstanceName;
				psInstanceName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psInstanceName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psInstanceName);

			//****************** Language ******************
			PropertySpec psLanguage = null;
			try
			{
				psLanguage = new PropertySpec(Resources.Properties.Property_Server_Language_Text
														, typeof(string)
														, Resources.Properties.Property_Server_Language_Category
														, Resources.Properties.Property_Server_Language_Description);

				propertyTable[Resources.Properties.Property_Server_Language_Text] = server.Information.Language;
				psLanguage.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psLanguage.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psLanguage);

			//****************** ComputerName ******************
			PropertySpec psComputerName = null;
			try
			{
				psComputerName = new PropertySpec(Resources.Properties.Property_Server_ComputerName_Text
														, typeof(string)
														, Resources.Properties.Property_Server_ComputerName_Category
														, Resources.Properties.Property_Server_ComputerName_Description);

				propertyTable[Resources.Properties.Property_Server_ComputerName_Text] = System.Environment.MachineName;
				psComputerName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psComputerName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psComputerName);

			//****************** Platform ******************
			PropertySpec psPlatform = null;
			try
			{
				psPlatform = new PropertySpec(Resources.Properties.Property_Server_Platform_Text
														, typeof(string)
														, Resources.Properties.Property_Server_Platform_Category
														, Resources.Properties.Property_Server_Platform_Description);

				propertyTable[Resources.Properties.Property_Server_Platform_Text] = server.Information.Platform;
				psPlatform.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psPlatform.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psPlatform);

			//****************** OperatingSystem ******************
			PropertySpec psOperatingSystem = null;
			try
			{
				psOperatingSystem = new PropertySpec(Resources.Properties.Property_Server_OperatingSystem_Text
														, typeof(string)
														, Resources.Properties.Property_Server_OperatingSystem_Category
														, Resources.Properties.Property_Server_OperatingSystem_Description);

				propertyTable[Resources.Properties.Property_Server_OperatingSystem_Text] = server.Information.OSVersion;
				psOperatingSystem.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOperatingSystem.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOperatingSystem);

			//****************** Processor ******************
			PropertySpec psProcessor = null;
			try
			{
				psProcessor = new PropertySpec(Resources.Properties.Property_Server_Processor_Text
														, typeof(string)
														, Resources.Properties.Property_Server_Processor_Category
														, Resources.Properties.Property_Server_Processor_Description);

				propertyTable[Resources.Properties.Property_Server_Processor_Text] = Convert.ToString(server.Information.Processors);
				psProcessor.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProcessor.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProcessor);

			//****************** OSMemory ******************
			PropertySpec psOSMemory = null;
			try
			{
				psOSMemory = new PropertySpec(Resources.Properties.Property_Server_OSMemory_Text
														, typeof(string)
														, Resources.Properties.Property_Server_OSMemory_Category
														, Resources.Properties.Property_Server_OSMemory_Description);

				propertyTable[Resources.Properties.Property_Server_OSMemory_Text] = Convert.ToString(server.Information.PhysicalMemory);
				psOSMemory.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOSMemory.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOSMemory);
			return propertyTable;
		}
		internal static PropertyTable GetDatabaseProperties(Database database)
		{
			if (database == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "database"));

			PropertyTable propertyTable = new PropertyTable();
			ServiceBroker sb = database.ServiceBroker;
			
			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_Database_Name_Text
														, typeof(string)
														, Resources.Properties.Property_Database_Name_Category
														, Resources.Properties.Property_Database_Name_Description);

				propertyTable[Resources.Properties.Property_Database_Name_Text] = database.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** Owner ******************
			PropertySpec psOwner = null;
			try
			{
				psOwner = new PropertySpec(Resources.Properties.Property_Database_Owner_Text
														, typeof(string)
														, Resources.Properties.Property_Database_Owner_Category
														, Resources.Properties.Property_Database_Owner_Description);

				propertyTable[Resources.Properties.Property_Database_Owner_Text] = database.Owner;
				psOwner.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOwner.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOwner);

			//****************** Size ******************
			PropertySpec psSize = null;
			try
			{
				psSize = new PropertySpec(Resources.Properties.Property_Database_Size_Text
														, typeof(string)
														, Resources.Properties.Property_Database_Size_Category
														, Resources.Properties.Property_Database_Size_Description);

				propertyTable[Resources.Properties.Property_Database_Size_Text] = database.Size;
				psSize.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psSize.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psSize);

			//****************** SpaceAvailable ******************
			PropertySpec psSpaceAvailable = null;
			try
			{
				psSpaceAvailable = new PropertySpec(Resources.Properties.Property_Database_SpaceAvailable_Text
														, typeof(string)
														, Resources.Properties.Property_Database_SpaceAvailable_Category
														, Resources.Properties.Property_Database_SpaceAvailable_Description);

				propertyTable[Resources.Properties.Property_Database_SpaceAvailable_Text] = database.SpaceAvailable;
				psSpaceAvailable.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psSpaceAvailable.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psSpaceAvailable);


			DataSet dsDatabaseProp = ServiceBrokerManager.GetDataBaseProperties(database);
			if (dsDatabaseProp != null)
			{
				if (dsDatabaseProp.Tables[ServiceBrokerConstants.DatabaseProperties] != null)
				{
					bool sbEnable = Convert.ToBoolean(dsDatabaseProp.Tables[ServiceBrokerConstants.DatabaseProperties].Rows[0][SqlColumnsConstant.Database_IsBrokerEnabled]);
					bool hasMasterKey = Convert.ToBoolean(dsDatabaseProp.Tables[ServiceBrokerConstants.DatabaseProperties].Rows[0][SqlColumnsConstant.Database_IsMasterKeyEncryptedByServer]);

					//****************** ServiceBrokerEnabled ******************
					PropertySpec psServiceBrokerEnabled = null;
					try
					{
						psServiceBrokerEnabled = new PropertySpec(Resources.Properties.Property_Database_ServiceBrokerEnabled_Text
																, typeof(bool)
																, Resources.Properties.Property_Database_ServiceBrokerEnabled_Category
																, Resources.Properties.Property_Database_ServiceBrokerEnabled_Description);

						propertyTable[Resources.Properties.Property_Database_ServiceBrokerEnabled_Text] = sbEnable;
						psServiceBrokerEnabled.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
					}
					catch (PropertyCannotBeRetrievedException)
					{
						psServiceBrokerEnabled.Attributes = new Attribute[] { BrowsableAttribute.No };
					}
					propertyTable.Properties.Add(psServiceBrokerEnabled);

					//****************** Master Key Enabled ******************
					PropertySpec psMasterKeyEnabled = null;
					try
					{
						psMasterKeyEnabled = new PropertySpec(Resources.Properties.Property_Database_MasterKeyEnabled_Text
																, typeof(bool)
																, Resources.Properties.Property_Database_MasterKeyEnabled_Category
																, Resources.Properties.Property_Database_MasterKeyEnabled_Description);

						propertyTable[Resources.Properties.Property_Database_MasterKeyEnabled_Text] = hasMasterKey;
						psMasterKeyEnabled.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
					}
					catch (PropertyCannotBeRetrievedException)
					{
						psMasterKeyEnabled.Attributes = new Attribute[] { BrowsableAttribute.No };
					}
					propertyTable.Properties.Add(psMasterKeyEnabled);
				}
			}
			
			//****************** MessageTypes ******************
			PropertySpec psMessageTypes = null;
			try
			{
				psMessageTypes = new PropertySpec(Resources.Properties.Property_Database_MessageTypes_Text
														, typeof(string)
														, Resources.Properties.Property_Database_MessageTypes_Category
														, Resources.Properties.Property_Database_MessageTypes_Description);

				propertyTable[Resources.Properties.Property_Database_MessageTypes_Text] = sb.MessageTypes.Count;
				psMessageTypes.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psMessageTypes.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psMessageTypes);

			//****************** Contracts ******************
			PropertySpec psContracts = null;
			try
			{
				psContracts = new PropertySpec(Resources.Properties.Property_Database_Contracts_Text
														, typeof(string)
														, Resources.Properties.Property_Database_Contracts_Category
														, Resources.Properties.Property_Database_Contracts_Description);

				propertyTable[Resources.Properties.Property_Database_Contracts_Text] = sb.ServiceContracts.Count;
				psContracts.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psContracts.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psContracts);

			//****************** Queues ******************
			PropertySpec psQueues = null;
			try
			{
				psQueues = new PropertySpec(Resources.Properties.Property_Database_Queues_Text
														, typeof(string)
														, Resources.Properties.Property_Database_Queues_Category
														, Resources.Properties.Property_Database_Queues_Description);

				propertyTable[Resources.Properties.Property_Database_Queues_Text] = sb.Queues.Count;
				psQueues.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psQueues.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psQueues);

			//****************** Services ******************
			PropertySpec psServices = null;
			try
			{
				psServices = new PropertySpec(Resources.Properties.Property_Database_Services_Text
														, typeof(string)
														, Resources.Properties.Property_Database_Services_Category
														, Resources.Properties.Property_Database_Services_Description);

				propertyTable[Resources.Properties.Property_Database_Services_Text] = sb.Services.Count;
				psServices.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psServices.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psServices);

			//****************** Routes ******************
			PropertySpec psRoutes = null;
			try
			{
				psRoutes = new PropertySpec(Resources.Properties.Property_Database_Routes_Text
														, typeof(string)
														, Resources.Properties.Property_Database_Routes_Category
														, Resources.Properties.Property_Database_Routes_Description);

				propertyTable[Resources.Properties.Property_Database_Routes_Text] = sb.Routes.Count;
				psRoutes.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psRoutes.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psRoutes);

			//****************** RemoteServiceBindings ******************
			PropertySpec psRemoteServiceBindings = null;
			try
			{
				psRemoteServiceBindings = new PropertySpec(Resources.Properties.Property_Database_RemoteServiceBindings_Text
														, typeof(string)
														, Resources.Properties.Property_Database_RemoteServiceBindings_Category
														, Resources.Properties.Property_Database_RemoteServiceBindings_Description);

				propertyTable[Resources.Properties.Property_Database_RemoteServiceBindings_Text] = sb.RemoteServiceBindings.Count;
				psRemoteServiceBindings.Attributes = new Attribute[] { ReadOnlyAttribute.Yes};;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psRemoteServiceBindings.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psRemoteServiceBindings);

			return propertyTable;
		}
		internal static PropertyTable GetLoginProperties(Login login)
		{
			if (login == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "login"));

			PropertyTable propertyTable = new PropertyTable();
			
			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_Login_Name_Text
														, typeof(string)
														, Resources.Properties.Property_Login_Name_Category
														, Resources.Properties.Property_Login_Name_Description);

				propertyTable[Resources.Properties.Property_Login_Name_Text] = login.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** Certificate ******************
			PropertySpec psCertificate = null;
			try
			{
				psCertificate = new PropertySpec(Resources.Properties.Property_Login_Certificate_Text
														, typeof(string)
														, Resources.Properties.Property_Login_Certificate_Category
														, Resources.Properties.Property_Login_Certificate_Description);

				propertyTable[Resources.Properties.Property_Login_Certificate_Text] = login.Certificate;
				psCertificate.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psCertificate.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psCertificate);

			//****************** Asymmetric Key ******************
			PropertySpec psAsymmetricKey = null;
			try
			{
				psAsymmetricKey = new PropertySpec(Resources.Properties.Property_Login_AsymmetricKey_Text
														, typeof(string)
														, Resources.Properties.Property_Login_AsymmetricKey_Category
														, Resources.Properties.Property_Login_AsymmetricKey_Description);

				propertyTable[Resources.Properties.Property_Login_AsymmetricKey_Text] = login.AsymmetricKey;
				psAsymmetricKey.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psAsymmetricKey.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psAsymmetricKey);

			//****************** IsLocked ******************
			PropertySpec psIsLocked =null;
			try
			{
				psIsLocked = new PropertySpec(Resources.Properties.Property_Login_IsLocked_Text
														, typeof(bool)
														, Resources.Properties.Property_Login_IsLocked_Category
														, Resources.Properties.Property_Login_IsLocked_Description);
				
				propertyTable[Resources.Properties.Property_Login_IsLocked_Text] = login.IsLocked;
				psIsLocked.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psIsLocked.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psIsLocked);

			//****************** Is Disabled ******************
			PropertySpec psIsDisabled =null;
			try
			{
				psIsDisabled = new PropertySpec(Resources.Properties.Property_Login_IsDisabled_Text
														, typeof(bool)
														, Resources.Properties.Property_Login_IsDisabled_Category
														, Resources.Properties.Property_Login_IsDisabled_Description);

				propertyTable[Resources.Properties.Property_Login_IsDisabled_Text] = login.IsDisabled;
				psIsDisabled.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psIsDisabled.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psIsDisabled);


			//****************** Credential ******************
			PropertySpec psCredential = null;
			try
			{
				psCredential = new PropertySpec(Resources.Properties.Property_Login_Credential_Text 
														, typeof(string)
														, Resources.Properties.Property_Login_Credential_Category
														, Resources.Properties.Property_Login_Credential_Description);

				propertyTable[Resources.Properties.Property_Login_Credential_Text] = login.Credential;
				psCredential.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psCredential.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psCredential);

			//****************** CreateDate ******************
			PropertySpec psCreateDate = null;
			try
			{
				psCreateDate = new PropertySpec(Resources.Properties.Property_Login_CreateDate_Text
														, typeof(DateTime)
														, Resources.Properties.Property_Login_CreateDate_Category
														, Resources.Properties.Property_Login_CreateDate_Description);

				propertyTable[Resources.Properties.Property_Login_CreateDate_Text] = login.CreateDate;
				psCreateDate.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psCreateDate.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psCreateDate);


			//****************** DateLastModified ******************
			PropertySpec psDateLastModified = null;
			try
			{
				psDateLastModified = new PropertySpec(Resources.Properties.Property_Login_DateLastModified_Text
														, typeof(DateTime)
														, Resources.Properties.Property_Login_DateLastModified_Category
														, Resources.Properties.Property_Login_DateLastModified_Description);

				propertyTable[Resources.Properties.Property_Login_DateLastModified_Text] = login.DateLastModified;
				psDateLastModified.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psDateLastModified.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psDateLastModified);


			//****************** DefaultDatabase ******************
			PropertySpec psDefaultDatabase = null;
			try
			{
				psDefaultDatabase = new PropertySpec(Resources.Properties.Property_Login_DefaultDatabase_Text
														, typeof(string)
														, Resources.Properties.Property_Login_DefaultDatabase_Category
														, Resources.Properties.Property_Login_DefaultDatabase_Description);

				propertyTable[Resources.Properties.Property_Login_DefaultDatabase_Text] = login.DefaultDatabase;
				psDefaultDatabase.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psDefaultDatabase.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psDefaultDatabase);


			//****************** DenyWindowsLogin ******************
			PropertySpec psDenyWindowsLogin = null;
			try
			{
				psDenyWindowsLogin = new PropertySpec(Resources.Properties.Property_Login_DenyWindowsLogin_Text
														, typeof(bool)
														, Resources.Properties.Property_Login_DenyWindowsLogin_Category
														, Resources.Properties.Property_Login_DenyWindowsLogin_Description);

				propertyTable[Resources.Properties.Property_Login_DenyWindowsLogin_Text] = login.DenyWindowsLogin;
				psDenyWindowsLogin.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psDenyWindowsLogin.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psDenyWindowsLogin);

			//****************** LoginType ******************
			PropertySpec psLoginType = null;
			try
			{
				psLoginType = new PropertySpec(Resources.Properties.Property_Login_LoginType_Text
														, typeof(LoginType)
														, Resources.Properties.Property_Login_LoginType_Category
														, Resources.Properties.Property_Login_LoginType_Description);

				propertyTable[Resources.Properties.Property_Login_LoginType_Text] = login.LoginType;
				psLoginType.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psLoginType.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psLoginType);


			//****************** WindowsLoginAccessType ******************
			PropertySpec psWindowsLoginAccessType = null;
			try
			{
				psWindowsLoginAccessType = new PropertySpec(Resources.Properties.Property_Login_WindowsLoginAccessType_Text
														, typeof(WindowsLoginAccessType)
														, Resources.Properties.Property_Login_WindowsLoginAccessType_Category
														, Resources.Properties.Property_Login_WindowsLoginAccessType_Description);

				propertyTable[Resources.Properties.Property_Login_WindowsLoginAccessType_Text] = login.WindowsLoginAccessType;
				psWindowsLoginAccessType.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psWindowsLoginAccessType.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psWindowsLoginAccessType);


			//****************** IsPasswordExpired ******************
			PropertySpec psIsPasswordExpired = null;
			try
			{
				psIsPasswordExpired = new PropertySpec(Resources.Properties.Property_Login_IsPasswordExpired_Text
														, typeof(bool)
														, Resources.Properties.Property_Login_IsPasswordExpired_Category
														, Resources.Properties.Property_Login_IsPasswordExpired_Description);

				propertyTable[Resources.Properties.Property_Login_IsPasswordExpired_Text] = login.IsPasswordExpired;
				psIsPasswordExpired.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psIsPasswordExpired.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psIsPasswordExpired);

			//****************** PasswordExpirationEnabled ******************
			PropertySpec psPasswordExpirationEnabled = null;
			try
			{
				psPasswordExpirationEnabled = new PropertySpec(Resources.Properties.Property_Login_PasswordExpirationEnabled_Text
														, typeof(bool)
														, Resources.Properties.Property_Login_PasswordExpirationEnabled_Category
														, Resources.Properties.Property_Login_PasswordExpirationEnabled_Description);

				propertyTable[Resources.Properties.Property_Login_PasswordExpirationEnabled_Text] = login.PasswordExpirationEnabled;
				psPasswordExpirationEnabled.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psPasswordExpirationEnabled.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psPasswordExpirationEnabled);

			//****************** PasswordPolicyEnforced ******************
			PropertySpec psPasswordPolicyEnforced = null;
			try
			{
				psPasswordPolicyEnforced = new PropertySpec(Resources.Properties.Property_Login_PasswordPolicyEnforced_Text
														, typeof(bool)
														, Resources.Properties.Property_Login_PasswordPolicyEnforced_Category
														, Resources.Properties.Property_Login_PasswordPolicyEnforced_Description);

				propertyTable[Resources.Properties.Property_Login_PasswordPolicyEnforced_Text] = login.PasswordPolicyEnforced;
				psPasswordPolicyEnforced.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psPasswordPolicyEnforced.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psPasswordPolicyEnforced);

			return propertyTable;
		}
		internal static PropertyTable GetDatabaseUserProperties(User user)
		{
			if (user == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "user"));

			PropertyTable propertyTable = new PropertyTable();

			//****************** Certificate ******************
			PropertySpec psCertificate = null;
			try
			{
				psCertificate = new PropertySpec(Resources.Properties.Property_User_Certificate_Text
														, typeof(string)
														, Resources.Properties.Property_User_Certificate_Category
														, Resources.Properties.Property_User_Certificate_Description);

				propertyTable[Resources.Properties.Property_User_Certificate_Text] = user.Certificate;
				psCertificate.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psCertificate.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psCertificate);

			//****************** AsymmetricKey ******************
			PropertySpec psAsymmetricKey = null;
			try
			{
				psAsymmetricKey = new PropertySpec(Resources.Properties.Property_User_AsymmetricKey_Text
														, typeof(string)
														, Resources.Properties.Property_User_AsymmetricKey_Category
														, Resources.Properties.Property_User_AsymmetricKey_Description);

				propertyTable[Resources.Properties.Property_User_AsymmetricKey_Text] = user.AsymmetricKey;
				psAsymmetricKey.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psAsymmetricKey.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psAsymmetricKey);

			//****************** CreateDate ******************
			PropertySpec psCreateDate = null;
			try
			{
				psCreateDate = new PropertySpec(Resources.Properties.Property_User_CreateDate_Text
														, typeof(DateTime)
														, Resources.Properties.Property_User_CreateDate_Category
														, Resources.Properties.Property_User_CreateDate_Description);

				propertyTable[Resources.Properties.Property_User_CreateDate_Text] = user.CreateDate;
				psCreateDate.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psCreateDate.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psCreateDate);

			//****************** DateLastModified ******************
			PropertySpec psDateLastModified = null;
			try
			{
				psDateLastModified = new PropertySpec(Resources.Properties.Property_User_DateLastModified_Text
														, typeof(DateTime)
														, Resources.Properties.Property_User_DateLastModified_Category
														, Resources.Properties.Property_User_DateLastModified_Description);

				propertyTable[Resources.Properties.Property_User_DateLastModified_Text] = user.DateLastModified;
				psDateLastModified.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psDateLastModified.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psDateLastModified);

			//****************** DefaultSchema ******************
			PropertySpec psDefaultSchema = null;
			try
			{
				psDefaultSchema = new PropertySpec(Resources.Properties.Property_User_DefaultSchema_Text
														, typeof(string)
														, Resources.Properties.Property_User_DefaultSchema_Category
														, Resources.Properties.Property_User_DefaultSchema_Description);

				propertyTable[Resources.Properties.Property_User_DefaultSchema_Text] = user.DefaultSchema;
				psDefaultSchema.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psDefaultSchema.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psDefaultSchema);

			//****************** Login ******************
			PropertySpec psLogin = null;
			try
			{
				psLogin = new PropertySpec(Resources.Properties.Property_User_Login_Text
														, typeof(string)
														, Resources.Properties.Property_User_Login_Category
														, Resources.Properties.Property_User_Login_Description);

				propertyTable[Resources.Properties.Property_User_Login_Text] = user.Login;
				psLogin.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psLogin.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psLogin);

			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_User_Name_Text
														, typeof(string)
														, Resources.Properties.Property_User_Name_Category
														, Resources.Properties.Property_User_Name_Description);

				propertyTable[Resources.Properties.Property_User_Name_Text] = user.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** LoginType ******************
			PropertySpec psLoginType = null;
			try
			{
				psLoginType = new PropertySpec(Resources.Properties.Property_User_LoginType_Text
														, typeof(LoginType)
														, Resources.Properties.Property_User_LoginType_Category
														, Resources.Properties.Property_User_LoginType_Description);

				propertyTable[Resources.Properties.Property_User_LoginType_Text] = user.LoginType;
				psLoginType.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psLoginType.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psLoginType);

			//****************** UserType ******************
			PropertySpec psUserType = null;
			try
			{
				psUserType = new PropertySpec(Resources.Properties.Property_User_UserType_Text
														, typeof(UserType)
														, Resources.Properties.Property_User_UserType_Category
														, Resources.Properties.Property_User_UserType_Description);

				propertyTable[Resources.Properties.Property_User_UserType_Text] = user.UserType;
				psUserType.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psUserType.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psUserType);

			return propertyTable;
		}
		internal static PropertyTable GetCertificateProperties(Certificate cert)
		{
			if (cert == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "cert"));

			PropertyTable propertyTable = new PropertyTable();

			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_Certificate_Name_Text
														, typeof(string)
														, Resources.Properties.Property_Certificate_Name_Category
														, Resources.Properties.Property_Certificate_Name_Description);

				propertyTable[Resources.Properties.Property_Certificate_Name_Text] = cert.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** Owner ******************
			PropertySpec psOwner = null;
			try
			{
				psOwner = new PropertySpec(Resources.Properties.Property_Certificate_Owner_Text
														, typeof(string)
														, Resources.Properties.Property_Certificate_Owner_Category
														, Resources.Properties.Property_Certificate_Owner_Description);

				propertyTable[Resources.Properties.Property_Certificate_Owner_Text] = cert.Owner;
				psOwner.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOwner.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOwner);

			//****************** StartDate ******************
			PropertySpec psStartDate = null;
			try
			{
				psStartDate = new PropertySpec(Resources.Properties.Property_Certificate_StartDate_Text
														, typeof(string)
														, Resources.Properties.Property_Certificate_StartDate_Category
														, Resources.Properties.Property_Certificate_StartDate_Description);

				propertyTable[Resources.Properties.Property_Certificate_StartDate_Text] = cert.StartDate;
				psStartDate.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psStartDate.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psStartDate);

			//****************** ExpirationDate ******************
			PropertySpec psExpirationDate = null;
			try
			{
				psExpirationDate = new PropertySpec(Resources.Properties.Property_Certificate_ExpirationDate_Text
														, typeof(string)
														, Resources.Properties.Property_Certificate_ExpirationDate_Category
														, Resources.Properties.Property_Certificate_ExpirationDate_Description);

				propertyTable[Resources.Properties.Property_Certificate_ExpirationDate_Text] = cert.ExpirationDate;
				psExpirationDate.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psExpirationDate.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psExpirationDate);

			//****************** Issuer ******************
			PropertySpec psIssuer = null;
			try
			{
				psIssuer = new PropertySpec(Resources.Properties.Property_Certificate_Issuer_Text
														, typeof(string)
														, Resources.Properties.Property_Certificate_Issuer_Category
														, Resources.Properties.Property_Certificate_Issuer_Description);

				propertyTable[Resources.Properties.Property_Certificate_Issuer_Text] = cert.Issuer;
				psIssuer.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psIssuer.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psIssuer);

			//****************** Subject ******************
			PropertySpec psSubject = null;
			try
			{
				psSubject = new PropertySpec(Resources.Properties.Property_Certificate_Subject_Text
														, typeof(string)
														, Resources.Properties.Property_Certificate_Subject_Category
														, Resources.Properties.Property_Certificate_Subject_Description);

				propertyTable[Resources.Properties.Property_Certificate_Subject_Text] = cert.Subject;
				psSubject.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psSubject.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psSubject);

			//****************** ActiveForServiceBrokerDialog ******************
			PropertySpec psActiveForServiceBrokerDialog = null;
			try
			{
				psActiveForServiceBrokerDialog = new PropertySpec(Resources.Properties.Property_Certificate_ActiveForServiceBrokerDialog_Text
														, typeof(string)
														, Resources.Properties.Property_Certificate_ActiveForServiceBrokerDialog_Category
														, Resources.Properties.Property_Certificate_ActiveForServiceBrokerDialog_Description);

				propertyTable[Resources.Properties.Property_Certificate_ActiveForServiceBrokerDialog_Text] = cert.ActiveForServiceBrokerDialog;
				psActiveForServiceBrokerDialog.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psActiveForServiceBrokerDialog.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psActiveForServiceBrokerDialog);

			//****************** PrivateKeyEncryptionType ******************
			PropertySpec psPrivateKeyEncryptionType = null;
			try
			{
				psPrivateKeyEncryptionType = new PropertySpec(Resources.Properties.Property_Certificate_PrivateKeyEncryptionType_Text
														, typeof(string)
														, Resources.Properties.Property_Certificate_PrivateKeyEncryptionType_Category
														, Resources.Properties.Property_Certificate_PrivateKeyEncryptionType_Description);

				propertyTable[Resources.Properties.Property_Certificate_PrivateKeyEncryptionType_Text] = cert.PrivateKeyEncryptionType;
				psPrivateKeyEncryptionType.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psPrivateKeyEncryptionType.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psPrivateKeyEncryptionType);

			//****************** Serial ******************
			PropertySpec psSerial = null;
			try
			{
				psSerial = new PropertySpec(Resources.Properties.Property_Certificate_Serial_Text
														, typeof(string)
														, Resources.Properties.Property_Certificate_Serial_Category
														, Resources.Properties.Property_Certificate_Serial_Description);

				propertyTable[Resources.Properties.Property_Certificate_Serial_Text] = cert.Serial;
				psSerial.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psSerial.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psSerial);


			return propertyTable;
		}
		internal static PropertyTable GetConnectionProperties(Server server)
		{
			if (server == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "server"));

			PropertyTable propertyTable = new PropertyTable();

			//****************** AuthenticationMethod ******************
			PropertySpec psAuthenticationMethod = null;
			try
			{
				psAuthenticationMethod = new PropertySpec(Resources.Properties.Property_Connection_AuthenticationMethod_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_AuthenticationMethod_Category
														, Resources.Properties.Property_Connection_AuthenticationMethod_Description);

				if (server.ConnectionContext.LoginSecure)
					propertyTable[Resources.Properties.Property_Connection_AuthenticationMethod_Text] = Digitaldeposit.Ssbm.Common.Enums.SqlServerAuthenticationMode.Windows.ToString("g");
				else
					propertyTable[Resources.Properties.Property_Connection_AuthenticationMethod_Text] = Digitaldeposit.Ssbm.Common.Enums.SqlServerAuthenticationMode.Mixed.ToString("g");

				psAuthenticationMethod.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psAuthenticationMethod.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psAuthenticationMethod);

			//****************** UserName ******************
			PropertySpec psUserName = null;
			try
			{
				psUserName = new PropertySpec(Resources.Properties.Property_Connection_UserName_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_UserName_Category
														, Resources.Properties.Property_Connection_UserName_Description);

				propertyTable[Resources.Properties.Property_Connection_UserName_Text] = server.ConnectionContext.TrueLogin;

				psUserName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psUserName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psUserName);

			//****************** Database ******************
			PropertySpec psDatabase = null;
			try
			{
				psDatabase = new PropertySpec(Resources.Properties.Property_Connection_Database_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_Database_Category
														, Resources.Properties.Property_Connection_Database_Description);

				propertyTable[Resources.Properties.Property_Connection_Database_Text] = server.ConnectionContext.DatabaseName;

				psDatabase.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psDatabase.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psDatabase);

			//****************** NetworkProtocol ******************
			PropertySpec psNetworkProtocol = null;
			try
			{
				psNetworkProtocol = new PropertySpec(Resources.Properties.Property_Connection_NetworkProtocol_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_NetworkProtocol_Category
														, Resources.Properties.Property_Connection_NetworkProtocol_Description);

				propertyTable[Resources.Properties.Property_Connection_NetworkProtocol_Text] = server.ConnectionContext.NetworkProtocol.ToString();

				psNetworkProtocol.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psNetworkProtocol.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psNetworkProtocol);

			//****************** NetworkPacketSize ******************
			PropertySpec psNetworkPacketSize = null;
			try
			{
				psNetworkPacketSize = new PropertySpec(Resources.Properties.Property_Connection_NetworkPacketSize_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_NetworkPacketSize_Category
														, Resources.Properties.Property_Connection_NetworkPacketSize_Description);

				propertyTable[Resources.Properties.Property_Connection_NetworkPacketSize_Text] = Convert.ToString(server.ConnectionContext.PacketSize);

				psNetworkPacketSize.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psNetworkPacketSize.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psNetworkPacketSize);

			//****************** ConnectionTimeout ******************
			PropertySpec psConnectionTimeout = null;
			try
			{
				psConnectionTimeout = new PropertySpec(Resources.Properties.Property_Connection_ConnectionTimeout_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_ConnectionTimeout_Category
														, Resources.Properties.Property_Connection_ConnectionTimeout_Description);

				propertyTable[Resources.Properties.Property_Connection_ConnectionTimeout_Text] = Convert.ToString(server.ConnectionContext.ConnectTimeout);

				psConnectionTimeout.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psConnectionTimeout.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psConnectionTimeout);

			//****************** Encrypted ******************
			PropertySpec psEncrypted = null;
			try
			{
				psEncrypted = new PropertySpec(Resources.Properties.Property_Connection_Encrypted_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_Encrypted_Category
														, Resources.Properties.Property_Connection_Encrypted_Description);

				propertyTable[Resources.Properties.Property_Connection_Encrypted_Text] = Convert.ToString(server.ConnectionContext.EncryptConnection);

				psEncrypted.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psEncrypted.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psEncrypted);

			//****************** ProductName ******************
			PropertySpec psProductName = null;
			try
			{
				psProductName = new PropertySpec(Resources.Properties.Property_Connection_ProductName_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_ProductName_Category
														, Resources.Properties.Property_Connection_ProductName_Description);

				propertyTable[Resources.Properties.Property_Connection_ProductName_Text] = server.Information.Product;

				psProductName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProductName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProductName);

			//****************** ProductVersion ******************
			PropertySpec psProductVersion = null;
			try
			{
				psProductVersion = new PropertySpec(Resources.Properties.Property_Connection_ProductVersion_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_ProductVersion_Category
														, Resources.Properties.Property_Connection_ProductVersion_Description);

				propertyTable[Resources.Properties.Property_Connection_ProductVersion_Text] = server.Information.Version.ToString();

				psProductVersion.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProductVersion.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProductVersion);

			//****************** InstanceName ******************
			PropertySpec psInstanceName = null;
			try
			{
				psInstanceName = new PropertySpec(Resources.Properties.Property_Connection_InstanceName_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_InstanceName_Category
														, Resources.Properties.Property_Connection_InstanceName_Description);

				propertyTable[Resources.Properties.Property_Connection_InstanceName_Text] = server.InstanceName;
				psInstanceName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psInstanceName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psInstanceName);

			//****************** Language ******************
			PropertySpec psLanguage = null;
			try
			{
				psLanguage = new PropertySpec(Resources.Properties.Property_Connection_Language_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_Language_Category
														, Resources.Properties.Property_Connection_Language_Description);

				propertyTable[Resources.Properties.Property_Connection_Language_Text] = server.Information.Language;
				psLanguage.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psLanguage.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psLanguage);

			//****************** ComputerName ******************
			PropertySpec psComputerName = null;
			try
			{
				psComputerName = new PropertySpec(Resources.Properties.Property_Connection_ComputerName_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_ComputerName_Category
														, Resources.Properties.Property_Connection_ComputerName_Description);

				propertyTable[Resources.Properties.Property_Connection_ComputerName_Text] = System.Environment.MachineName;
				psComputerName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psComputerName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psComputerName);

			//****************** Platform ******************
			PropertySpec psPlatform = null;
			try
			{
				psPlatform = new PropertySpec(Resources.Properties.Property_Connection_Platform_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_Platform_Category
														, Resources.Properties.Property_Connection_Platform_Description);

				propertyTable[Resources.Properties.Property_Connection_Platform_Text] = server.Information.Platform;
				psPlatform.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psPlatform.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psPlatform);

			//****************** OperatingSystem ******************
			PropertySpec psOperatingSystem = null;
			try
			{
				psOperatingSystem = new PropertySpec(Resources.Properties.Property_Connection_OperatingSystem_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_OperatingSystem_Category
														, Resources.Properties.Property_Connection_OperatingSystem_Description);

				propertyTable[Resources.Properties.Property_Connection_OperatingSystem_Text] = server.Information.OSVersion;
				psOperatingSystem.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOperatingSystem.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOperatingSystem);

			//****************** Processor ******************
			PropertySpec psProcessor = null;
			try
			{
				psProcessor = new PropertySpec(Resources.Properties.Property_Connection_Processor_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_Processor_Category
														, Resources.Properties.Property_Connection_Processor_Description);

				propertyTable[Resources.Properties.Property_Connection_Processor_Text] = Convert.ToString(server.Information.Processors);
				psProcessor.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProcessor.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProcessor);

			//****************** OSMemory ******************
			PropertySpec psOSMemory = null;
			try
			{
				psOSMemory = new PropertySpec(Resources.Properties.Property_Connection_OSMemory_Text
														, typeof(string)
														, Resources.Properties.Property_Connection_OSMemory_Category
														, Resources.Properties.Property_Connection_OSMemory_Description);

				propertyTable[Resources.Properties.Property_Connection_OSMemory_Text] = Convert.ToString(server.Information.PhysicalMemory);
				psOSMemory.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOSMemory.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOSMemory);


			return propertyTable;
		}
		internal static PropertyTable GetMessageTypeProperties(MessageType messageType)
		{
			if (messageType == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "messageType"));

			PropertyTable propertyTable = new PropertyTable();

			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_MessageType_Name_Text
														, typeof(string)
														, Resources.Properties.Property_MessageType_Name_Category
														, Resources.Properties.Property_MessageType_Name_Description);

				propertyTable[Resources.Properties.Property_MessageType_Name_Text] = messageType.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** Owner ******************
			PropertySpec psOwner = null;
			try
			{
				psOwner = new PropertySpec(Resources.Properties.Property_MessageType_Owner_Text
														, typeof(string)
														, Resources.Properties.Property_MessageType_Owner_Category
														, Resources.Properties.Property_MessageType_Owner_Description);

				propertyTable[Resources.Properties.Property_MessageType_Owner_Text] = messageType.Owner;
				psOwner.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOwner.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOwner);

			//****************** Parent ******************
			PropertySpec psParent = null;
			try
			{
				psParent = new PropertySpec(Resources.Properties.Property_MessageType_Parent_Text
														, typeof(string)
														, Resources.Properties.Property_MessageType_Parent_Category
														, Resources.Properties.Property_MessageType_Parent_Description);

				propertyTable[Resources.Properties.Property_MessageType_Parent_Text] = messageType.Parent;
				psParent.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psParent.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psParent);

			//****************** MessageTypeValidation ******************
			PropertySpec psMessageTypeValidation = null;
			try
			{
				psMessageTypeValidation = new PropertySpec(Resources.Properties.Property_MessageType_MessageTypeValidation_Text
														, typeof(MessageTypeValidation)
														, Resources.Properties.Property_MessageType_MessageTypeValidation_Category
														, Resources.Properties.Property_MessageType_MessageTypeValidation_Description);

				propertyTable[Resources.Properties.Property_MessageType_MessageTypeValidation_Text] = messageType.MessageTypeValidation;
				psMessageTypeValidation.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psMessageTypeValidation.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psMessageTypeValidation);

			//****************** ValidationXmlSchemaCollection ******************
			PropertySpec psValidationXmlSchemaCollection = null;
			try
			{
				psValidationXmlSchemaCollection = new PropertySpec(Resources.Properties.Property_MessageType_ValidationXmlSchemaCollection_Text
														, typeof(string)
														, Resources.Properties.Property_MessageType_ValidationXmlSchemaCollection_Category
														, Resources.Properties.Property_MessageType_ValidationXmlSchemaCollection_Description);

				propertyTable[Resources.Properties.Property_MessageType_ValidationXmlSchemaCollection_Text] = messageType.ValidationXmlSchemaCollection;
				psValidationXmlSchemaCollection.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psValidationXmlSchemaCollection.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psValidationXmlSchemaCollection);


			return propertyTable;
		}
		internal static PropertyTable GetContractProperties(ServiceContract contract)
		{
			if (contract == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "contract"));

			PropertyTable propertyTable = new PropertyTable();

			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_Contract_Name_Text
														, typeof(string)
														, Resources.Properties.Property_Contract_Name_Category
														, Resources.Properties.Property_Contract_Name_Description);

				propertyTable[Resources.Properties.Property_Contract_Name_Text] = contract.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** Authorization ******************
			PropertySpec psAuthorization = null;
			try
			{
				psAuthorization = new PropertySpec(Resources.Properties.Property_Contract_Authorization_Text
														, typeof(string)
														, Resources.Properties.Property_Contract_Authorization_Category
														, Resources.Properties.Property_Contract_Authorization_Description);

				propertyTable[Resources.Properties.Property_Contract_Authorization_Text] = contract.Owner;
				psAuthorization.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psAuthorization.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psAuthorization);

			//****************** MessageTypeMapping ******************
			PropertySpec psMessageTypeMapping = null;
			try
			{
				psMessageTypeMapping = new PropertySpec(Resources.Properties.Property_Contract_MessageTypeMapping_Text
														, typeof(MessageTypeMappingCollection)
														, Resources.Properties.Property_Contract_MessageTypeMapping_Category
														, Resources.Properties.Property_Contract_MessageTypeMapping_Description);

				propertyTable[Resources.Properties.Property_Contract_MessageTypeMapping_Text] = contract.MessageTypeMappings;
				psMessageTypeMapping.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psMessageTypeMapping.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psMessageTypeMapping);

			return propertyTable;
		}
		internal static PropertyTable GetQueueProperties(ServiceQueue queue)
		{
			if (queue == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "queue"));

			PropertyTable propertyTable = new PropertyTable();


			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_Queue_Name_Text
														, typeof(string)
														, Resources.Properties.Property_Queue_Name_Category
														, Resources.Properties.Property_Queue_Name_Description);

				propertyTable[Resources.Properties.Property_Queue_Name_Text] = queue.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** Status ******************
			PropertySpec psStatus = null;
			try
			{
				psStatus = new PropertySpec(Resources.Properties.Property_Queue_Status_Text
														, typeof(bool)
														, Resources.Properties.Property_Queue_Status_Category
														, Resources.Properties.Property_Queue_Status_Description);

				propertyTable[Resources.Properties.Property_Queue_Status_Text] = queue.IsEnqueueEnabled;
				psStatus.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psStatus.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psStatus);

			//****************** Retention ******************
			PropertySpec psRetention = null;
			try
			{
				psRetention = new PropertySpec(Resources.Properties.Property_Queue_Retention_Text
														, typeof(bool)
														, Resources.Properties.Property_Queue_Retention_Category
														, Resources.Properties.Property_Queue_Retention_Description);

				propertyTable[Resources.Properties.Property_Queue_Retention_Text] = queue.IsRetentionEnabled;
				psRetention.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psRetention.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psRetention);

			//****************** Activation ******************
			PropertySpec psActivation = null;
			try
			{
				psActivation = new PropertySpec(Resources.Properties.Property_Queue_Activation_Text
														, typeof(bool)
														, Resources.Properties.Property_Queue_Activation_Category
														, Resources.Properties.Property_Queue_Activation_Description);

				propertyTable[Resources.Properties.Property_Queue_Activation_Text] = queue.IsActivationEnabled;
				psActivation.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psActivation.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psActivation);

			//****************** ProcedureDatabase ******************
			PropertySpec psProcedureDatabase = null;
			try
			{
				psProcedureDatabase = new PropertySpec(Resources.Properties.Property_Queue_ProcedureDatabase_Text
														, typeof(string)
														, Resources.Properties.Property_Queue_ProcedureDatabase_Category
														, Resources.Properties.Property_Queue_ProcedureDatabase_Description);

				propertyTable[Resources.Properties.Property_Queue_ProcedureDatabase_Text] = queue.ProcedureDatabase;
				psProcedureDatabase.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProcedureDatabase.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProcedureDatabase);

			//****************** ProcedureName ******************
			PropertySpec psProcedureName = null;
			try
			{
				psProcedureName = new PropertySpec(Resources.Properties.Property_Queue_ProcedureName_Text
														, typeof(string)
														, Resources.Properties.Property_Queue_ProcedureName_Category
														, Resources.Properties.Property_Queue_ProcedureName_Description);

				propertyTable[Resources.Properties.Property_Queue_ProcedureName_Text] = queue.ProcedureName;
				psProcedureName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProcedureName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProcedureName);

			//****************** ProcedureSchema ******************
			PropertySpec psProcedureSchema = null;
			try
			{
				psProcedureSchema = new PropertySpec(Resources.Properties.Property_Queue_ProcedureSchema_Text
														, typeof(string)
														, Resources.Properties.Property_Queue_ProcedureSchema_Category
														, Resources.Properties.Property_Queue_ProcedureSchema_Description);

				propertyTable[Resources.Properties.Property_Queue_ProcedureSchema_Text] = queue.ProcedureSchema;
				psProcedureSchema.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProcedureSchema.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProcedureSchema);

			//****************** MaximumReaders ******************
			PropertySpec psMaximumReaders = null;
			try
			{
				psMaximumReaders = new PropertySpec(Resources.Properties.Property_Queue_MaximumReaders_Text
														, typeof(short)
														, Resources.Properties.Property_Queue_MaximumReaders_Category
														, Resources.Properties.Property_Queue_MaximumReaders_Description);

				propertyTable[Resources.Properties.Property_Queue_MaximumReaders_Text] = queue.MaxReaders;
				psMaximumReaders.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psMaximumReaders.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psMaximumReaders);

			//****************** ExecuteAs ******************
			PropertySpec psExecuteAs = null;
			try
			{
				psExecuteAs = new PropertySpec(Resources.Properties.Property_Queue_ExecuteAs_Text
														, typeof(string)
														, Resources.Properties.Property_Queue_ExecuteAs_Category
														, Resources.Properties.Property_Queue_ExecuteAs_Description);

				propertyTable[Resources.Properties.Property_Queue_ExecuteAs_Text] = queue.ExecutionContextPrincipal;
				psExecuteAs.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psExecuteAs.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psExecuteAs);

			//****************** FileGroup ******************
			PropertySpec psFileGroup = null;
			try
			{
				psFileGroup = new PropertySpec(Resources.Properties.Property_Queue_FileGroup_Text
														, typeof(string)
														, Resources.Properties.Property_Queue_FileGroup_Category
														, Resources.Properties.Property_Queue_FileGroup_Description);

				propertyTable[Resources.Properties.Property_Queue_FileGroup_Text] = queue.FileGroup;
				psFileGroup.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psFileGroup.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psFileGroup);

			//****************** CreatedDate ******************
			PropertySpec psCreatedDate = null;
			try
			{
				psCreatedDate = new PropertySpec(Resources.Properties.Property_Queue_CreatedDate_Text
														, typeof(DateTime)
														, Resources.Properties.Property_Queue_CreatedDate_Category
														, Resources.Properties.Property_Queue_CreatedDate_Description);

				propertyTable[Resources.Properties.Property_Queue_CreatedDate_Text] = queue.CreateDate;
				psCreatedDate.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psCreatedDate.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psCreatedDate);

			//****************** LastModified ******************
			PropertySpec psLastModified = null;
			try
			{
				psLastModified = new PropertySpec(Resources.Properties.Property_Queue_LastModified_Text
														, typeof(DateTime)
														, Resources.Properties.Property_Queue_LastModified_Category
														, Resources.Properties.Property_Queue_LastModified_Description);

				propertyTable[Resources.Properties.Property_Queue_LastModified_Text] = queue.DateLastModified;
				psLastModified.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psLastModified.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psLastModified);

			//****************** RowCount ******************
			PropertySpec psRowCount = null;
			try
			{
				psRowCount = new PropertySpec(Resources.Properties.Property_Queue_RowCount_Text
														, typeof(long)
														, Resources.Properties.Property_Queue_RowCount_Category
														, Resources.Properties.Property_Queue_RowCount_Description);

				propertyTable[Resources.Properties.Property_Queue_RowCount_Text] = queue.RowCount;
				psRowCount.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psRowCount.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psRowCount);

			return propertyTable;
		}
		internal static PropertyTable GetServiceProperties(BrokerService service)
		{
			if (service == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "service"));

			PropertyTable propertyTable = new PropertyTable();

			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_Service_Name_Text
														, typeof(string)
														, Resources.Properties.Property_Service_Name_Category
														, Resources.Properties.Property_Service_Name_Description);

				propertyTable[Resources.Properties.Property_Service_Name_Text] = service.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** Owner ******************
			PropertySpec psOwner = null;
			try
			{
				psOwner = new PropertySpec(Resources.Properties.Property_Service_Owner_Text
														, typeof(string)
														, Resources.Properties.Property_Service_Owner_Category
														, Resources.Properties.Property_Service_Owner_Description);

				propertyTable[Resources.Properties.Property_Service_Owner_Text] = service.Owner;
				psOwner.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOwner.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOwner);

			//****************** QueueName ******************
			PropertySpec psQueueName = null;
			try
			{
				psQueueName = new PropertySpec(Resources.Properties.Property_Service_QueueName_Text
														, typeof(string)
														, Resources.Properties.Property_Service_QueueName_Category
														, Resources.Properties.Property_Service_QueueName_Description);

				propertyTable[Resources.Properties.Property_Service_QueueName_Text] = service.QueueName;
				psQueueName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psQueueName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psQueueName);

			//****************** QueueSchema ******************
			PropertySpec psQueueSchema = null;
			try
			{
				psQueueSchema = new PropertySpec(Resources.Properties.Property_Service_QueueSchema_Text
														, typeof(string)
														, Resources.Properties.Property_Service_QueueSchema_Category
														, Resources.Properties.Property_Service_QueueSchema_Description);

				propertyTable[Resources.Properties.Property_Service_QueueSchema_Text] = service.QueueSchema;
				psQueueSchema.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psQueueSchema.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psQueueSchema);

			//****************** Contracts ******************
			PropertySpec psContracts = null;
			try
			{
				psContracts = new PropertySpec(Resources.Properties.Property_Service_Contracts_Text
														, typeof(ServiceContractMappingCollection)
														, Resources.Properties.Property_Service_Contracts_Category
														, Resources.Properties.Property_Service_Contracts_Description);

				propertyTable[Resources.Properties.Property_Service_Contracts_Text] = service.ServiceContractMappings;
				psContracts.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psContracts.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psContracts);


			return propertyTable;
		}
		internal static PropertyTable GetRouteProperties(ServiceRoute route)
		{
			if (route == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "route"));

			PropertyTable propertyTable = new PropertyTable();

			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_Route_Name_Text
														, typeof(string)
														, Resources.Properties.Property_Route_Name_Category
														, Resources.Properties.Property_Route_Name_Description);

				propertyTable[Resources.Properties.Property_Route_Name_Text] = route.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);
			
			//****************** Owner ******************
			PropertySpec psOwner = null;
			try
			{
				psOwner = new PropertySpec(Resources.Properties.Property_Route_Owner_Text
														, typeof(string)
														, Resources.Properties.Property_Route_Owner_Category
														, Resources.Properties.Property_Route_Owner_Description);

				propertyTable[Resources.Properties.Property_Route_Owner_Text] = route.Owner;
				psOwner.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOwner.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOwner);
			
			//****************** RemoteService ******************
			PropertySpec psRemoteService = null;
			try
			{
				psRemoteService = new PropertySpec(Resources.Properties.Property_Route_RemoteService_Text
														, typeof(string)
														, Resources.Properties.Property_Route_RemoteService_Category
														, Resources.Properties.Property_Route_RemoteService_Description);

				propertyTable[Resources.Properties.Property_Route_RemoteService_Text] = route.RemoteService;
				psRemoteService.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psRemoteService.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psRemoteService);
			
			//****************** MirrorAddress ******************
			PropertySpec psMirrorAddress = null;
			try
			{
				psMirrorAddress = new PropertySpec(Resources.Properties.Property_Route_MirrorAddress_Text
														, typeof(string)
														, Resources.Properties.Property_Route_MirrorAddress_Category
														, Resources.Properties.Property_Route_MirrorAddress_Description);

				propertyTable[Resources.Properties.Property_Route_MirrorAddress_Text] = route.MirrorAddress;
				psMirrorAddress.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psMirrorAddress.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psMirrorAddress);
			
			//****************** BrokerInstance ******************
			PropertySpec psBrokerInstance = null;
			try
			{
				psBrokerInstance = new PropertySpec(Resources.Properties.Property_Route_BrokerInstance_Text
														, typeof(string)
														, Resources.Properties.Property_Route_BrokerInstance_Category
														, Resources.Properties.Property_Route_BrokerInstance_Description);

				propertyTable[Resources.Properties.Property_Route_BrokerInstance_Text] = route.BrokerInstance;
				psBrokerInstance.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psBrokerInstance.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psBrokerInstance);
			
			//****************** ExpirationDate ******************
			PropertySpec psExpirationDate = null;
			try
			{
				psExpirationDate = new PropertySpec(Resources.Properties.Property_Route_ExpirationDate_Text
														, typeof(DateTime)
														, Resources.Properties.Property_Route_ExpirationDate_Category
														, Resources.Properties.Property_Route_ExpirationDate_Description);

				propertyTable[Resources.Properties.Property_Route_ExpirationDate_Text] = route.ExpirationDate;
				psExpirationDate.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psExpirationDate.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psExpirationDate);
			
			//****************** Address ******************
			PropertySpec psAddress = null;
			try
			{
				psAddress = new PropertySpec(Resources.Properties.Property_Route_Address_Text
														, typeof(string)
														, Resources.Properties.Property_Route_Address_Category
														, Resources.Properties.Property_Route_Address_Description);

				propertyTable[Resources.Properties.Property_Route_Address_Text] = route.Address;
				psAddress.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psAddress.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psAddress);
			return propertyTable;
		}
		internal static PropertyTable GetRemoteServiceBindingProperties(RemoteServiceBinding remoteServiceBinding)
		{
			if (remoteServiceBinding == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "remoteServiceBinding"));

			PropertyTable propertyTable = new PropertyTable();

			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_RemoteServiceBinding_Name_Text
														, typeof(string)
														, Resources.Properties.Property_RemoteServiceBinding_Name_Category
														, Resources.Properties.Property_RemoteServiceBinding_Name_Description);

				propertyTable[Resources.Properties.Property_RemoteServiceBinding_Name_Text] = remoteServiceBinding.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** Owner ******************
			PropertySpec psOwner = null;
			try
			{
				psOwner = new PropertySpec(Resources.Properties.Property_RemoteServiceBinding_Owner_Text
														, typeof(string)
														, Resources.Properties.Property_RemoteServiceBinding_Owner_Category
														, Resources.Properties.Property_RemoteServiceBinding_Owner_Description);

				propertyTable[Resources.Properties.Property_RemoteServiceBinding_Owner_Text] = remoteServiceBinding.Owner;
				psOwner.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOwner.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOwner);

			//****************** CertificateUser ******************
			PropertySpec psCertificateUser = null;
			try
			{
				psCertificateUser = new PropertySpec(Resources.Properties.Property_RemoteServiceBinding_CertificateUser_Text
														, typeof(string)
														, Resources.Properties.Property_RemoteServiceBinding_CertificateUser_Category
														, Resources.Properties.Property_RemoteServiceBinding_CertificateUser_Description);

				propertyTable[Resources.Properties.Property_RemoteServiceBinding_CertificateUser_Text] = remoteServiceBinding.CertificateUser;
				psCertificateUser.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psCertificateUser.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psCertificateUser);

			//****************** RemoteService ******************
			PropertySpec psRemoteService = null;
			
			try
			{
				psRemoteService = new PropertySpec(Resources.Properties.Property_RemoteServiceBinding_RemoteService_Text
														, typeof(string)
														, Resources.Properties.Property_RemoteServiceBinding_RemoteService_Category
														, Resources.Properties.Property_RemoteServiceBinding_RemoteService_Description);

				propertyTable[Resources.Properties.Property_RemoteServiceBinding_RemoteService_Text] = remoteServiceBinding.RemoteService;
				psRemoteService.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psRemoteService.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psRemoteService);

			//****************** IsAnonymous ******************
			PropertySpec psIsAnonymous = null;
			try
			{
				psIsAnonymous = new PropertySpec(Resources.Properties.Property_RemoteServiceBinding_IsAnonymous_Text
														, typeof(string)
														, Resources.Properties.Property_RemoteServiceBinding_IsAnonymous_Category
														, Resources.Properties.Property_RemoteServiceBinding_IsAnonymous_Description);

				propertyTable[Resources.Properties.Property_RemoteServiceBinding_IsAnonymous_Text] = remoteServiceBinding.IsAnonymous;
				psIsAnonymous.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psIsAnonymous.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psIsAnonymous);

			return propertyTable;
		}
		internal static PropertyTable GetXmlSchemaCollectionProperties(XmlSchemaCollection xmlSchemaCol)
		{
			if (xmlSchemaCol == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "xmlSchemaCol"));

			PropertyTable propertyTable = new PropertyTable();

			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_XmlSchemaCol_Name_Text
														, typeof(string)
														, Resources.Properties.Property_XmlSchemaCol_Name_Category
														, Resources.Properties.Property_XmlSchemaCol_Name_Description);

				propertyTable[Resources.Properties.Property_XmlSchemaCol_Name_Text] = xmlSchemaCol.Name;
				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** Schema ******************
			PropertySpec psSchema = null;
			try
			{
				psSchema = new PropertySpec(Resources.Properties.Property_XmlSchemaCol_Schema_Text
														, typeof(string)
														, Resources.Properties.Property_XmlSchemaCol_Schema_Category
														, Resources.Properties.Property_XmlSchemaCol_Schema_Description);

				propertyTable[Resources.Properties.Property_XmlSchemaCol_Schema_Text] = xmlSchemaCol.Schema;
				psSchema.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psSchema.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psSchema);


			//****************** Text ******************
			PropertySpec psText = null;
			try
			{
				psText = new PropertySpec(Resources.Properties.Property_XmlSchemaCol_Text_Text
														, typeof(string)
														, Resources.Properties.Property_XmlSchemaCol_Text_Category
														, Resources.Properties.Property_XmlSchemaCol_Text_Description);

				propertyTable[Resources.Properties.Property_XmlSchemaCol_Text_Text] = xmlSchemaCol.Text;
				psText.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psText.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psText);


			//****************** CreateDate ******************
			PropertySpec psCreateDate = null;
			try
			{
				psCreateDate = new PropertySpec(Resources.Properties.Property_XmlSchemaCol_Name_Text
														, typeof(DateTime)
														, Resources.Properties.Property_XmlSchemaCol_CreateDate_Category
														, Resources.Properties.Property_XmlSchemaCol_CreateDate_Description);

				propertyTable[Resources.Properties.Property_XmlSchemaCol_CreateDate_Text] = xmlSchemaCol.CreateDate;
				psCreateDate.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psCreateDate.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psCreateDate);


			//****************** LastModified ******************
			PropertySpec psLastModified = null;
			try
			{
				psLastModified = new PropertySpec(Resources.Properties.Property_XmlSchemaCol_LastModified_Text
														, typeof(DateTime)
														, Resources.Properties.Property_XmlSchemaCol_LastModified_Category
														, Resources.Properties.Property_XmlSchemaCol_LastModified_Description);

				propertyTable[Resources.Properties.Property_XmlSchemaCol_LastModified_Text] = xmlSchemaCol.DateLastModified;
				psLastModified.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psLastModified.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psLastModified);

			return propertyTable;
		}
		internal static PropertyTable GetEndpointProperties(Endpoint endPoint)
		{
			if (endPoint == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "endPoint"));

			PropertyTable propertyTable = new PropertyTable();

			//****************** Name ******************
			PropertySpec psName = null;
			try
			{
				psName = new PropertySpec(Resources.Properties.Property_EndPoint_Name_Text
														, typeof(string)
														, Resources.Properties.Property_EndPoint_Name_Category
														, Resources.Properties.Property_EndPoint_Name_Description);

				propertyTable[Resources.Properties.Property_EndPoint_Name_Text] = endPoint.Name;

				psName.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psName.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psName);

			//****************** Owner ******************
			PropertySpec psOwner = null;
			try
			{
				psOwner = new PropertySpec(Resources.Properties.Property_EndPoint_Owner_Text
														, typeof(string)
														, Resources.Properties.Property_EndPoint_Owner_Category
														, Resources.Properties.Property_EndPoint_Owner_Description);

				propertyTable[Resources.Properties.Property_EndPoint_Owner_Text] = endPoint.Owner;

				psOwner.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psOwner.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psOwner);
			//****************** Protocol ******************
			PropertySpec psProtocol = null;
			try
			{
				psProtocol = new PropertySpec(Resources.Properties.Property_EndPoint_Protocol_Text
														, typeof(Protocol)
														, Resources.Properties.Property_EndPoint_Protocol_Category
														, Resources.Properties.Property_EndPoint_Protocol_Description);

				propertyTable[Resources.Properties.Property_EndPoint_Protocol_Text] = endPoint.Protocol;

				psProtocol.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProtocol.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProtocol);
			//****************** ProtocolType ******************
			PropertySpec psProtocolType = null;
			try
			{
				psProtocolType = new PropertySpec(Resources.Properties.Property_EndPoint_ProtocolType_Text
														, typeof(ProtocolType)
														, Resources.Properties.Property_EndPoint_ProtocolType_Category
														, Resources.Properties.Property_EndPoint_ProtocolType_Description);

				propertyTable[Resources.Properties.Property_EndPoint_ProtocolType_Text] = endPoint.ProtocolType;

				psProtocolType.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psProtocolType.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psProtocolType);
			//****************** EndpointState ******************
			PropertySpec psEndpointState = null;
			try
			{
				psEndpointState = new PropertySpec(Resources.Properties.Property_EndPoint_EndpointState_Text
														, typeof(EndpointState)
														, Resources.Properties.Property_EndPoint_EndpointState_Category
														, Resources.Properties.Property_EndPoint_EndpointState_Description);

				propertyTable[Resources.Properties.Property_EndPoint_EndpointState_Text] = endPoint.EndpointState;

				psEndpointState.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psEndpointState.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psEndpointState);
			//****************** EndpointType ******************
			PropertySpec psEndpointType = null;
			try
			{
				psEndpointType = new PropertySpec(Resources.Properties.Property_EndPoint_EndpointType_Text
														, typeof(EndpointType)
														, Resources.Properties.Property_EndPoint_EndpointType_Category
														, Resources.Properties.Property_EndPoint_EndpointType_Description);

				propertyTable[Resources.Properties.Property_EndPoint_EndpointType_Text] = endPoint.EndpointType;

				psEndpointType.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psEndpointType.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psEndpointType);
			//****************** IsAdminEndpoint ******************
			PropertySpec psIsAdminEndpoint = null;
			try
			{
				psIsAdminEndpoint = new PropertySpec(Resources.Properties.Property_EndPoint_IsAdminEndpoint_Text
														, typeof(bool)
														, Resources.Properties.Property_EndPoint_IsAdminEndpoint_Category
														, Resources.Properties.Property_EndPoint_IsAdminEndpoint_Description);

				propertyTable[Resources.Properties.Property_EndPoint_IsAdminEndpoint_Text] = endPoint.IsAdminEndpoint;

				psIsAdminEndpoint.Attributes = new Attribute[] { ReadOnlyAttribute.Yes };
			}
			catch (PropertyCannotBeRetrievedException)
			{
				psIsAdminEndpoint.Attributes = new Attribute[] { BrowsableAttribute.No };
			}
			propertyTable.Properties.Add(psIsAdminEndpoint);

			return propertyTable;
		}
		#endregion
	}
}
