//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;

using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.MessageBox;

using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.PropertyMaintenance;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;

namespace Digitaldeposit.Ssbm.Screens
{
	/// <summary>
	/// Used whenever there is a requirement to displays rows of data in the screen.
	/// Example: Transmission queue data, Messages in the queue etc.
	/// </summary>
	public partial class DataDisplayerForm : Form
	{
		#region Constructor
		/// <summary>
		/// Used whenever there is a requirement to displays rows of data in the screen.
		/// Has some properties to set, since its used in multiple places.
		/// </summary>
		/// <param name="windowName">string to represent the name of the window.</param>
		/// <param name="header">Header text</param>
		/// <param name="databaseName">Name of the datbase as string</param>
		/// <param name="titleImageKey">Imagekey, </param>
		/// <param name="dataDisplayerType">Need to display the type (DataDisplayerType enum), so that data grid columns can be set </param>
		public DataDisplayerForm(string windowName, string header, string databaseName, string titleImageKey, DataDisplayerType dataDisplayerType)
		{
			InitializeComponent();
			ImageList imageList = ImageListHelper.GetServiceBrokerImages();

			this.Text = windowName;
			
			lblTitle.Text = header;
			headerImage.Image = imageList.Images[titleImageKey];

			lblDatabaseName.Text = databaseName;
			dbImage.Image = imageList.Images[ProjectImagesConstants.Database];

			this.dgvMessagesInQueue.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.ColumnHeader;
		
			//Setup Grid Columns
			SetupGrid(dgvMessagesInQueue, dataDisplayerType);
			
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Used to update the property grid with corresponding data.
		/// </summary>
		/// <param name="ds">Dataset object with data in it.</param>
		/// <param name="datamember">Datamember to pick from the dataset.</param>
		public void UpdateMessagesGrid(DataSet ds, string datamember)
		{
			dgvMessagesInQueue.DataSource = ds;
			dgvMessagesInQueue.DataMember = datamember;
			dgvMessagesInQueue.AutoResizeColumns(DataGridViewAutoSizeColumnsMode.ColumnHeader);
		}
		#endregion

		#region Event Handlers
		private void Messages_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
		{
			DataGridView dgv = (DataGridView)sender;

			if (dgv.Columns[e.ColumnIndex].Name == SqlColumnsConstant.MessagesInQueue_MessageTypeName)
			{
				if (SystemObjects.IsSystemSSBObject(SsbmObjectType.MessageType,Convert.ToString(e.Value)))
				{
					e.CellStyle.ForeColor = UIConstants.SystemObjectsColor;
				}
			}
		}
		private static void SetupGrid(object sender, DataDisplayerType dataDisplayerType)
		{
			DataGridView dgv = (DataGridView)sender;

			dgv.AutoGenerateColumns = false;

											
			switch(dataDisplayerType)
			{
				case DataDisplayerType.QueueStatus:
					dgv.Columns.Add(SqlColumnsConstant.QueueName, Resources.DataGridHeaders.Queue_QueueName);
					dgv.Columns[SqlColumnsConstant.QueueName].DataPropertyName = SqlColumnsConstant.QueueName;

					dgv.Columns.Add(SqlColumnsConstant.ServiceName, Resources.DataGridHeaders.Queue_ServiceName);
					dgv.Columns[SqlColumnsConstant.ServiceName].DataPropertyName = SqlColumnsConstant.ServiceName;

					dgv.Columns.Add(SqlColumnsConstant.SchemaName, Resources.DataGridHeaders.Queue_SchemaName);
					dgv.Columns[SqlColumnsConstant.SchemaName].DataPropertyName = SqlColumnsConstant.SchemaName;

					dgv.Columns.Add(SqlColumnsConstant.QueueState, Resources.DataGridHeaders.Queue_QueueState);
					dgv.Columns[SqlColumnsConstant.QueueState].DataPropertyName = SqlColumnsConstant.QueueState;

					dgv.Columns.Add(SqlColumnsConstant.TasksWaiting, Resources.DataGridHeaders.Queue_TasksWaiting);
					dgv.Columns[SqlColumnsConstant.TasksWaiting].DataPropertyName = SqlColumnsConstant.TasksWaiting;

					dgv.Columns.Add(SqlColumnsConstant.LastActivatedTime, Resources.DataGridHeaders.Queue_LastActivatedTime);
					dgv.Columns[SqlColumnsConstant.LastActivatedTime].DataPropertyName = SqlColumnsConstant.LastActivatedTime;

					dgv.Columns.Add(SqlColumnsConstant.LastEmptyRowsetTime, Resources.DataGridHeaders.Queue_LastEmptyRowsetTime);
					dgv.Columns[SqlColumnsConstant.LastEmptyRowsetTime].DataPropertyName = SqlColumnsConstant.LastEmptyRowsetTime;

					dgv.Columns.Add(SqlColumnsConstant.TranMessageCount, Resources.DataGridHeaders.Queue_TranMessageCount);
					dgv.Columns[SqlColumnsConstant.TranMessageCount].DataPropertyName = SqlColumnsConstant.TranMessageCount;
	
					break;

				case DataDisplayerType.MessagesInQueue:
					
					dgv.Columns.Add(SqlColumnsConstant.MessagesInQueue_MessageTypeName, Resources.DataGridHeaders.Message_MessageTypeName);
					dgv.Columns[SqlColumnsConstant.MessagesInQueue_MessageTypeName].DataPropertyName = SqlColumnsConstant.MessagesInQueue_MessageTypeName;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInQueue_MessageSequenceNumber, Resources.DataGridHeaders.Message_MessageSequenceNumber);
					dgv.Columns[SqlColumnsConstant.MessagesInQueue_MessageSequenceNumber].DataPropertyName = SqlColumnsConstant.MessagesInQueue_MessageSequenceNumber;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInQueue_QueueOrder, Resources.DataGridHeaders.Message_QueueOrder);
					dgv.Columns[SqlColumnsConstant.MessagesInQueue_QueueOrder].DataPropertyName = SqlColumnsConstant.MessagesInQueue_QueueOrder;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInQueue_Validation, Resources.DataGridHeaders.Message_Validation);
					dgv.Columns[SqlColumnsConstant.MessagesInQueue_Validation].DataPropertyName = SqlColumnsConstant.MessagesInQueue_Validation;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInQueue_Status, Resources.DataGridHeaders.Message_Status);
					dgv.Columns[SqlColumnsConstant.MessagesInQueue_Status].DataPropertyName = SqlColumnsConstant.MessagesInQueue_Status;
					break;
				
				case DataDisplayerType.TransmissionQueue:

					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_ConversationHandle,Resources.DataGridHeaders.TransmissionQ_ConversationHandle);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_ConversationHandle].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_ConversationHandle;


					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_EnqueueTime,Resources.DataGridHeaders.TransmissionQ_EnqueueTime);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_EnqueueTime].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_EnqueueTime;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_FromServiceName,Resources.DataGridHeaders.TransmissionQ_FromServiceName);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_FromServiceName].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_FromServiceName;


					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_IsConversationError,Resources.DataGridHeaders.TransmissionQ_IsConversationError);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_IsConversationError].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_IsConversationError;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_IsEndOfDialog,Resources.DataGridHeaders.TransmissionQ_IsEndOfDialog);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_IsEndOfDialog].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_IsEndOfDialog;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_MessageSequenceNumber,Resources.DataGridHeaders.TransmissionQ_MessageSequenceNumber);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_MessageSequenceNumber].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_MessageSequenceNumber;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_MessageTypeName,Resources.DataGridHeaders.TransmissionQ_MessageTypeName);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_MessageTypeName].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_MessageTypeName;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_ServiceContractName,Resources.DataGridHeaders.TransmissionQ_ServiceContractName);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_ServiceContractName].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_ServiceContractName;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_ToBrokeInstance,Resources.DataGridHeaders.TransmissionQ_ToBrokeInstance);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_ToBrokeInstance].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_ToBrokeInstance;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_ToServiceName,Resources.DataGridHeaders.TransmissionQ_ToServiceName);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_ToServiceName].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_ToServiceName;

					dgv.Columns.Add(SqlColumnsConstant.MessagesInTransmissionQ_TransmissionStatus,Resources.DataGridHeaders.TransmissionQ_TransmissionStatus);
					dgv.Columns[SqlColumnsConstant.MessagesInTransmissionQ_TransmissionStatus].DataPropertyName = SqlColumnsConstant.MessagesInTransmissionQ_TransmissionStatus;

					break;
				case DataDisplayerType.ActivatedTask:
					dgv.Columns.Add(SqlColumnsConstant.Monitoring_AT_spid, SqlColumnsConstant.Monitoring_AT_spid);
					dgv.Columns[SqlColumnsConstant.Monitoring_AT_spid].DataPropertyName = SqlColumnsConstant.Monitoring_AT_spid;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_AT_database_id, SqlColumnsConstant.Monitoring_AT_database_id);
					dgv.Columns[SqlColumnsConstant.Monitoring_AT_database_id].DataPropertyName = SqlColumnsConstant.Monitoring_AT_database_id;
					
					dgv.Columns.Add(SqlColumnsConstant.Monitoring_AT_queue_id, SqlColumnsConstant.Monitoring_AT_queue_id);
					dgv.Columns[SqlColumnsConstant.Monitoring_AT_queue_id].DataPropertyName = SqlColumnsConstant.Monitoring_AT_queue_id;
					
					dgv.Columns.Add(SqlColumnsConstant.Monitoring_AT_procedure_name, SqlColumnsConstant.Monitoring_AT_procedure_name);
					dgv.Columns[SqlColumnsConstant.Monitoring_AT_procedure_name].DataPropertyName = SqlColumnsConstant.Monitoring_AT_procedure_name;
					
					dgv.Columns.Add(SqlColumnsConstant.Monitoring_AT_execute_as, SqlColumnsConstant.Monitoring_AT_execute_as);
					dgv.Columns[SqlColumnsConstant.Monitoring_AT_execute_as].DataPropertyName = SqlColumnsConstant.Monitoring_AT_execute_as;

					break;
				case DataDisplayerType.BrokerConnection:
					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_connection_id, SqlColumnsConstant.Monitoring_BC_connection_id);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_connection_id].DataPropertyName = SqlColumnsConstant.Monitoring_BC_connection_id;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_transport_stream_id, SqlColumnsConstant.Monitoring_BC_transport_stream_id);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_transport_stream_id].DataPropertyName = SqlColumnsConstant.Monitoring_BC_transport_stream_id;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_state, SqlColumnsConstant.Monitoring_BC_state);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_state].DataPropertyName = SqlColumnsConstant.Monitoring_BC_state;


					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_state_desc, SqlColumnsConstant.Monitoring_BC_state_desc);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_state_desc].DataPropertyName = SqlColumnsConstant.Monitoring_BC_state_desc;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_connect_time, SqlColumnsConstant.Monitoring_BC_connect_time);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_connect_time].DataPropertyName = SqlColumnsConstant.Monitoring_BC_connect_time;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_login_time, SqlColumnsConstant.Monitoring_BC_login_time);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_login_time].DataPropertyName = SqlColumnsConstant.Monitoring_BC_login_time;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_authentication_method, SqlColumnsConstant.Monitoring_BC_authentication_method);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_authentication_method].DataPropertyName = SqlColumnsConstant.Monitoring_BC_authentication_method;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_principal_name, SqlColumnsConstant.Monitoring_BC_principal_name);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_principal_name].DataPropertyName = SqlColumnsConstant.Monitoring_BC_principal_name;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_remote_user_name, SqlColumnsConstant.Monitoring_BC_remote_user_name);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_remote_user_name].DataPropertyName = SqlColumnsConstant.Monitoring_BC_remote_user_name;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_last_activity_time, SqlColumnsConstant.Monitoring_BC_last_activity_time);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_last_activity_time].DataPropertyName = SqlColumnsConstant.Monitoring_BC_last_activity_time;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_is_accept, SqlColumnsConstant.Monitoring_BC_is_accept);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_is_accept].DataPropertyName = SqlColumnsConstant.Monitoring_BC_is_accept;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_login_state, SqlColumnsConstant.Monitoring_BC_login_state);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_login_state].DataPropertyName = SqlColumnsConstant.Monitoring_BC_login_state;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_login_state_desc, SqlColumnsConstant.Monitoring_BC_login_state_desc);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_login_state_desc].DataPropertyName = SqlColumnsConstant.Monitoring_BC_login_state_desc;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_peer_certificate_id, SqlColumnsConstant.Monitoring_BC_peer_certificate_id);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_peer_certificate_id].DataPropertyName = SqlColumnsConstant.Monitoring_BC_peer_certificate_id;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_encryption_algorithm, SqlColumnsConstant.Monitoring_BC_encryption_algorithm);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_encryption_algorithm].DataPropertyName = SqlColumnsConstant.Monitoring_BC_encryption_algorithm;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_encryption_algorithm_desc, SqlColumnsConstant.Monitoring_BC_encryption_algorithm_desc);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_encryption_algorithm_desc].DataPropertyName = SqlColumnsConstant.Monitoring_BC_encryption_algorithm_desc;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_receives_posted, SqlColumnsConstant.Monitoring_BC_receives_posted);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_receives_posted].DataPropertyName = SqlColumnsConstant.Monitoring_BC_receives_posted;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_is_receive_flow_controlled, SqlColumnsConstant.Monitoring_BC_is_receive_flow_controlled);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_is_receive_flow_controlled].DataPropertyName = SqlColumnsConstant.Monitoring_BC_is_receive_flow_controlled;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_sends_posted, SqlColumnsConstant.Monitoring_BC_sends_posted);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_sends_posted].DataPropertyName = SqlColumnsConstant.Monitoring_BC_sends_posted;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_is_send_flow_controlled, SqlColumnsConstant.Monitoring_BC_is_send_flow_controlled);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_is_send_flow_controlled].DataPropertyName = SqlColumnsConstant.Monitoring_BC_is_send_flow_controlled;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_total_bytes_sent, SqlColumnsConstant.Monitoring_BC_total_bytes_sent);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_total_bytes_sent].DataPropertyName = SqlColumnsConstant.Monitoring_BC_total_bytes_sent;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_total_bytes_received, SqlColumnsConstant.Monitoring_BC_total_bytes_received);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_total_bytes_received].DataPropertyName = SqlColumnsConstant.Monitoring_BC_total_bytes_received;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_total_fragments_sent, SqlColumnsConstant.Monitoring_BC_total_fragments_sent);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_total_fragments_sent].DataPropertyName = SqlColumnsConstant.Monitoring_BC_total_fragments_sent;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_total_fragments_received, SqlColumnsConstant.Monitoring_BC_total_fragments_received);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_total_fragments_received].DataPropertyName = SqlColumnsConstant.Monitoring_BC_total_fragments_received;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_total_sends, SqlColumnsConstant.Monitoring_BC_total_sends);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_total_sends].DataPropertyName = SqlColumnsConstant.Monitoring_BC_total_sends;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_total_receives, SqlColumnsConstant.Monitoring_BC_total_receives);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_total_receives].DataPropertyName = SqlColumnsConstant.Monitoring_BC_total_receives;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_BC_peer_arbitration_id, SqlColumnsConstant.Monitoring_BC_peer_arbitration_id);
					dgv.Columns[SqlColumnsConstant.Monitoring_BC_peer_arbitration_id].DataPropertyName = SqlColumnsConstant.Monitoring_BC_peer_arbitration_id;
					break;
				case DataDisplayerType.ForwardedMessage:
					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_conversation_id, SqlColumnsConstant.Monitoring_FM_conversation_id);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_conversation_id].DataPropertyName = SqlColumnsConstant.Monitoring_FM_conversation_id;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_is_initiator, SqlColumnsConstant.Monitoring_FM_is_initiator);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_is_initiator].DataPropertyName = SqlColumnsConstant.Monitoring_FM_is_initiator;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_to_service_name, SqlColumnsConstant.Monitoring_FM_to_service_name);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_to_service_name].DataPropertyName = SqlColumnsConstant.Monitoring_FM_to_service_name;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_to_broker_instance, SqlColumnsConstant.Monitoring_FM_to_broker_instance);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_to_broker_instance].DataPropertyName = SqlColumnsConstant.Monitoring_FM_to_broker_instance;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_from_service_name, SqlColumnsConstant.Monitoring_FM_from_service_name);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_from_service_name].DataPropertyName = SqlColumnsConstant.Monitoring_FM_from_service_name;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_from_broker_instance, SqlColumnsConstant.Monitoring_FM_from_broker_instance);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_from_broker_instance].DataPropertyName = SqlColumnsConstant.Monitoring_FM_from_broker_instance;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_adjacent_broker_address, SqlColumnsConstant.Monitoring_FM_adjacent_broker_address);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_adjacent_broker_address].DataPropertyName = SqlColumnsConstant.Monitoring_FM_adjacent_broker_address;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_message_sequence_number, SqlColumnsConstant.Monitoring_FM_message_sequence_number);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_message_sequence_number].DataPropertyName = SqlColumnsConstant.Monitoring_FM_message_sequence_number;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_message_fragment_number, SqlColumnsConstant.Monitoring_FM_message_fragment_number);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_message_fragment_number].DataPropertyName = SqlColumnsConstant.Monitoring_FM_message_fragment_number;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_hops_remaining, SqlColumnsConstant.Monitoring_FM_hops_remaining);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_hops_remaining].DataPropertyName = SqlColumnsConstant.Monitoring_FM_hops_remaining;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_time_to_live, SqlColumnsConstant.Monitoring_FM_time_to_live);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_time_to_live].DataPropertyName = SqlColumnsConstant.Monitoring_FM_time_to_live;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_time_consumed, SqlColumnsConstant.Monitoring_FM_time_consumed);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_time_consumed].DataPropertyName = SqlColumnsConstant.Monitoring_FM_time_consumed;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_FM_message_id, SqlColumnsConstant.Monitoring_FM_message_id);
					dgv.Columns[SqlColumnsConstant.Monitoring_FM_message_id].DataPropertyName = SqlColumnsConstant.Monitoring_FM_message_id;


					break;
				case DataDisplayerType.QueueMonitor:
					dgv.Columns.Add(SqlColumnsConstant.Monitoring_QM_database_id, SqlColumnsConstant.Monitoring_QM_database_id);
					dgv.Columns[SqlColumnsConstant.Monitoring_QM_database_id].DataPropertyName = SqlColumnsConstant.Monitoring_QM_database_id;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_QM_last_activated_time, SqlColumnsConstant.Monitoring_QM_last_activated_time);
					dgv.Columns[SqlColumnsConstant.Monitoring_QM_last_activated_time].DataPropertyName = SqlColumnsConstant.Monitoring_QM_last_activated_time;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_QM_last_empty_rowset_time, SqlColumnsConstant.Monitoring_QM_last_empty_rowset_time);
					dgv.Columns[SqlColumnsConstant.Monitoring_QM_last_empty_rowset_time].DataPropertyName = SqlColumnsConstant.Monitoring_QM_last_empty_rowset_time;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_QM_queue_id, SqlColumnsConstant.Monitoring_QM_queue_id);
					dgv.Columns[SqlColumnsConstant.Monitoring_QM_queue_id].DataPropertyName = SqlColumnsConstant.Monitoring_QM_queue_id;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_QM_state, SqlColumnsConstant.Monitoring_QM_state);
					dgv.Columns[SqlColumnsConstant.Monitoring_QM_state].DataPropertyName = SqlColumnsConstant.Monitoring_QM_state;

					dgv.Columns.Add(SqlColumnsConstant.Monitoring_QM_tasks_waiting, SqlColumnsConstant.Monitoring_QM_tasks_waiting);
					dgv.Columns[SqlColumnsConstant.Monitoring_QM_tasks_waiting].DataPropertyName = SqlColumnsConstant.Monitoring_QM_tasks_waiting;

					break;
			}
		}
		#endregion
	}
}