//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;

namespace Digitaldeposit.Ssbm.Screens
{
	public partial class EndConversationWarningForm : Form
	{
		#region Field
		int _errorCode;
		string _errorDescription = string.Empty;
		#endregion

		#region Constructor
		/// <summary>
		/// Public Constructor to initialize warning window with correct text messages based on the error type like
		/// EndConversation, End Conversation with error, End Conversation with Clean up and purge conversations.
		/// </summary>
		/// <param name="endConversationType">EndConversationType to determine the End Conversation activity.</param>
		public EndConversationWarningForm(EndConversationType endConversationType)
		{
			InitializeComponent();

			if (endConversationType == EndConversationType.EndConversation)
			{
				this.lblTitle.Text = Digitaldeposit.Ssbm.Resources.EndConversation.EndConversation_Title;
				this.lblWarning.Text = Digitaldeposit.Ssbm.Resources.EndConversation.EndConversation_Warning;
				this.panelError.Enabled = false;

			}
			else if (endConversationType == EndConversationType.EndConversationWithCleanUp)
			{
				this.lblTitle.Text = Digitaldeposit.Ssbm.Resources.EndConversation.EndWithCleanUp_Title;
				this.lblWarning.Text = Digitaldeposit.Ssbm.Resources.EndConversation.EndWithCleanup_Warning;
				this.panelError.Enabled = false;
			}
			else if (endConversationType == EndConversationType.EndConversationWithError)
			{
				this.lblTitle.Text = Digitaldeposit.Ssbm.Resources.EndConversation.EndWithError_Title;
				this.lblWarning.Text = Digitaldeposit.Ssbm.Resources.EndConversation.EndConversationWithError_Warning;
				this.panelError.Enabled = true;
			}
			else if (endConversationType == EndConversationType.PurgeAllConversationInDB)
			{
				this.lblTitle.Text = Digitaldeposit.Ssbm.Resources.EndConversation.Purge_Title;
				this.lblWarning.Text = Digitaldeposit.Ssbm.Resources.EndConversation.Purge_Warning;
				this.panelError.Enabled = false;
			}

			this.lblConfirmation.Text = Digitaldeposit.Ssbm.Resources.EndConversation.EndConversation_Confirmation;

		}
		#endregion

		#region Properties
		/// <summary>
		/// Contains ErrorCode to End Conversation with Error
		/// </summary>
		public int ErrorCode
		{
			get { return _errorCode; }
		}

		/// <summary>
		/// Contains ErrorDescription to End Conversation with Error
		/// </summary>

		public string ErrorDescription
		{
			get { return _errorDescription; }
		}
		#endregion

		#region Event Handlers
		private void btnOK_Click(object sender, EventArgs e)
		{
			_errorCode = Convert.ToInt32(txtErrorCode.Value);
			_errorDescription = txtErrorDescription.Text;

			this.DialogResult = DialogResult.OK;
			this.Close();
		}
		private void btnCancel_Click(object sender, EventArgs e)
		{
			this.DialogResult = DialogResult.Cancel;
			this.Close();
		}
		#endregion
	}
}