//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################

using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;

using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Smo.Broker;
using Microsoft.SqlServer.MessageBox;

using Northwoods.Go;

using Digitaldeposit.Ssbm.PropertyMaintenance;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;
using Digitaldeposit.Ssbm.Diagram;
using Digitaldeposit.Ssbm.Constants;

namespace Digitaldeposit.Ssbm.Screens
{
	/// <summary>
	/// Class used to display the relational diagram between various objects.
	/// </summary>
	public partial class RelationalDiagramForm : Form
	{
		/// <summary>
		/// Class used to display the relational diagram between various objects.
		/// </summary>
		/// <param name="server">Server object</param>
		/// <param name="databaseName">Database name</param>
		/// <param name="serviceName">service name to start the graph.</param>
		public RelationalDiagramForm(Server server, string databaseName, string serviceName)
		{
			if (server == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "server"));

			InitializeComponent();

			ImageList imageList = ImageListHelper.GetServiceBrokerImages();

			//Setup the legend images
			pbService.Image = imageList.Images[ProjectImagesConstants.Service];
			pbQueue.Image = imageList.Images[ProjectImagesConstants.Queue];
			pbMessageType.Image = imageList.Images[ProjectImagesConstants.MessageType];
			pbContract.Image = imageList.Images[ProjectImagesConstants.Contract];
			pbActivationSproc.Image = imageList.Images[ProjectImagesConstants.ActivationSproc];


			headerImage.Image = imageList.Images[ProjectImagesConstants.Service];
			lblTitle.Text = lblTitle.Text + serviceName;

			Database db = server.Databases[databaseName];
			ServiceBroker sb = db.ServiceBroker;
			BrokerService service = sb.Services[serviceName];
			ServiceQueue queue = sb.Queues[service.QueueName];

			goView1.ImageList = imageList;

			Cursor cur = Cursor.Current;
			this.Cursor = Cursors.WaitCursor;
			
			CreateIconicRelationDiagram(sb, service, queue);
			
			this.Cursor = cur;
		}
		private void CreateIconicRelationDiagram(ServiceBroker sb, BrokerService service, ServiceQueue queue)
		{
			int gap = 60;
			int X = goView1.Left + gap + service.Name.Length;
			int Y = goView1.Height / 2;

			goView1.AllowLink = false;


			GoIconicNode gnService = MakeIconicNode(ProjectImagesConstants.Service, service.Name);
			gnService.Location = new PointF(X, Y);

			GoIconicNode gnQueue = MakeIconicNode(ProjectImagesConstants.Queue, service.QueueName);
			gnQueue.Location = new PointF(X + gap, Y - gap);


			//Connect Service to Queue
			Connector cnServiceQueue = new Connector(gnService.Port, gnQueue.Port, goView1);

			if (queue.ProcedureName.Length != 0)
			{
				GoIconicNode gnActSproc = MakeIconicNode(ProjectImagesConstants.ActivationSproc, queue.ProcedureName);
				gnActSproc.Location = new PointF(X + gap * 2, Y - gap * 2);
		
				//Connect Queue to Activation procdeure
				Connector cnQueueActSproc = new Connector(gnQueue.Port,gnActSproc.Port,goView1);
			}

			//Start building the diagram from contract
			CreateDiagramStartingFromContract(sb, service, queue, gnService, gap);
		}
		private void CreateDiagramStartingFromContract(ServiceBroker sb, BrokerService service, ServiceQueue queue,  GoIconicNode gnService, int gap)
		{
			float service_X,service_Y; 
			if (gnService == null)
			{
				service_X = goView1.Left + gap;
				service_Y = goView1.Height / 2;
			}
			else
			{
				service_X = gnService.Location.X;
				service_Y = gnService.Location.Y;
			}


			goView1.AllowLink = false;

			GoIconicNode gnContract, gnMessageType;
			GoIconicNode gnInitiator, gnTarget, gnInitiatorTarget;

			//To make the count easier
			ArrayList aInitiator = new ArrayList();
			ArrayList aTarget = new ArrayList();
			ArrayList aInitiatorTarget = new ArrayList();

			Connector linkServiceContract, linkContractMessageType, linkContractInitiator, linkContractTarget, linkContractInitTarg;

			float Contract_Y = service_Y + gap;
			foreach (ServiceContractMapping scm in service.ServiceContractMappings)
			{
				gnContract = MakeIconicNode(ProjectImagesConstants.Contract, scm.Name);
				gnContract.Location = new PointF(service_X + gap, Contract_Y);

				//Connect Service to Contract
				if(gnService != null)
					linkServiceContract = new Connector(gnService.Port, gnContract.Port, goView1);
			
				//Initiator, Target, Init/Target nodes to distinguish messagetypes.
				gnInitiator = MakeIconicNode(ProjectImagesConstants.MessageType, "Initiator");
				gnInitiator.Location = new PointF(gnContract.Location.X + gap * 2, gnContract.Location.Y - gap * 1);

				gnTarget = MakeIconicNode(ProjectImagesConstants.MessageType, "Target");
				gnTarget.Location = new PointF(gnContract.Location.X + gap * 2, gnContract.Location.Y);

				gnInitiatorTarget = MakeIconicNode(ProjectImagesConstants.MessageType, "Initiator-Target");



				//Connect Contract to Initiator,target and intiator/target
				linkContractInitiator = new Connector(gnContract.Port, gnInitiator.Port, goView1);
				linkContractTarget = new Connector(gnContract.Port, gnTarget.Port, goView1);
				linkContractInitTarg = new Connector(gnContract.Port, gnInitiatorTarget.Port, goView1);

				ServiceContract sc = sb.ServiceContracts[scm.Name];

				float messageType_Y = gnInitiator.Location.Y - gap;
				foreach (MessageTypeMapping mtm in sc.MessageTypeMappings)
				{
					gnMessageType = MakeIconicNode(ProjectImagesConstants.MessageType, mtm.Name);

					if (mtm.MessageSource == MessageSource.Initiator)
					{
						aInitiator.Add(gnMessageType);
						//gnMessageType.Location = new PointF(gnInitiator.Location.X + gap * 2, messageType_Y);

						//Connect Initiator to MessageType
						linkContractMessageType = new Connector(gnInitiator.Port, gnMessageType.Port, goView1);
					}

					if (mtm.MessageSource == MessageSource.Target)
					{
						aTarget.Add(gnMessageType);
						//gnMessageType.Location = new PointF(gnTarget.Location.X + gap * 2, messageType_Y);

						//Connect Target to MessageType
						linkContractMessageType = new Connector(gnTarget.Port, gnMessageType.Port, goView1);
					}

					if (mtm.MessageSource == MessageSource.InitiatorAndTarget)
					{
						aInitiatorTarget.Add(gnMessageType);
						//gnMessageType.Location = new PointF(gnInitiatorTarget.Location.X + gap * 2, messageType_Y);

						//Connect Initiator/Target to MessageType
						linkContractMessageType = new Connector(gnInitiatorTarget.Port, gnMessageType.Port, goView1);
					}

					messageType_Y += gap;
				}

				float mtY = gnInitiator.Location.Y;
				for (int i = aInitiator.Count - 1; i >= 0; i--)
				{
					GoIconicNode tempGnMessageType = (GoIconicNode)aInitiator[i];
					tempGnMessageType.Location = new PointF(gnInitiator.Location.X + gap * 2, mtY);
					mtY = mtY - gap;
				}

				mtY = gnTarget.Location.Y;
				for (int i = 0; i < aTarget.Count; i++)
				{
					GoIconicNode tempGnMessageType = (GoIconicNode)aTarget[i];
					tempGnMessageType.Location = new PointF(gnTarget.Location.X + gap * 2, mtY);
					mtY = mtY + gap;
				}

				if (aTarget.Count == 0)
					gnInitiatorTarget.Location = new PointF(gnContract.Location.X + gap * 2, gnContract.Location.Y + gap);
				else
					gnInitiatorTarget.Location = new PointF(gnContract.Location.X + gap * 2, mtY);

				mtY = gnInitiatorTarget.Location.Y;

				for (int i = 0; i < aInitiatorTarget.Count; i++)
				{
					GoIconicNode tempGnMessageType = (GoIconicNode)aInitiatorTarget[i];
					tempGnMessageType.Location = new PointF(gnInitiatorTarget.Location.X + gap * 2, mtY);//watch! mty is picked up from the last value of target
					mtY = tempGnMessageType.Location.Y + gap;
				}

				//Clear all the array lists
				aInitiator.Clear(); aTarget.Clear(); aInitiatorTarget.Clear();

				Contract_Y = gnInitiatorTarget.Location.Y + gap * 2;
			}
		}
		private GoIconicNode MakeIconicNode(string imageKey, string name)
		{
			GoIconicNode n = new GoIconicNode();
			ImageList imgList = ImageListHelper.GetServiceBrokerImages();

			int idx = imgList.Images.IndexOfKey(imageKey);

			n.ToolTipText = name;
			
			//Adjust the name length, so that it won't look horrible
			if (name.Length > 20)
			{
				name = name.Substring(0, 20)+ "....";
			}

			n.Initialize(null, idx, name);
			n.Editable = false;
			n.Label.Editable = false;
			n.Icon.Size = new SizeF(16, 16);
			n.Port.Bounds = n.Icon.Bounds;
			
			goView1.Document.Add(n);
			return n;
		}
	}
}