//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;
using System.Reflection;
using System.IO;

using Microsoft.SqlServer.MessageBox;
using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Smo.Broker;

using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.Manager;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;

namespace Digitaldeposit.Ssbm.Screens
{
	/// <summary>
	/// Used to send messages between conversation.
	/// </summary>
	public partial class SendMessageForm : Form
	{
		#region Fields
		Conversation g_Conversation;
		#endregion

		#region Constructors
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="db">Databse object</param>
		/// <param name="conversation">Conversation</param>
		public SendMessageForm(Database db, Conversation conversation)
		{
			if (db == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "db"));

			if (conversation == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.InvalidArgument, "conversation"));

			InitializeComponent();

			//initialize
			tvMessageType.ImageList = ImageListHelper.GetServiceBrokerImages(); ;
			g_Conversation = conversation;
			
			//Populate Message Type Tree view
			ServiceBroker sb = db.ServiceBroker;
			foreach (MessageType mt in sb.MessageTypes)
			{
				TreeNode tn = new TreeNode(mt.Name);
				tn.Name = mt.Name;
				tn.ImageKey = ProjectImagesConstants.MessageType;
				tn.SelectedImageKey = ProjectImagesConstants.MessageType;
				tvMessageType.Nodes.Add(tn);
			}

			//Assign all the required values
			this.lblConversationHandle.Text = Convert.ToString(conversation.ConversationHandle);

		}
		#endregion

		#region Event Handlers
		private void btnClose_Click(object sender, EventArgs e)
		{
			try
			{
				if (txtSelectedMessageType.Text.Length == 0)
					throw new InvalidDataException(Resources.Exceptions.SendMessage_InvalidMessageType);

				if(txtMessage.Text.Length == 0)
					throw new InvalidDataException(Resources.Exceptions.SendMessage_EmptyMessage);


				MemoryStream body = new MemoryStream(Encoding.ASCII.GetBytes(txtMessage.Text));
				Digitaldeposit.Ssbm.Manager.Message message = new Digitaldeposit.Ssbm.Manager.Message(txtSelectedMessageType.Text, body);

				g_Conversation.Send(message);

				this.DialogResult = DialogResult.OK;
				this.Close();
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		private void tvMessageType_AfterSelect(object sender, TreeViewEventArgs e)
		{
			txtSelectedMessageType.Text = tvMessageType.SelectedNode.Text;
		}
		private void btnCancel_Click(object sender, EventArgs e)
		{
			this.DialogResult = DialogResult.Cancel;
			this.Close();
		}
		#endregion
	}
}